{$d-}
program Toy(*Prolog*) ( input, output{, mc, si, so} ) ;
{$d+}

(* (C) COPYRIGHT 1983  --  Feliks Kluzniak, Stanislaw Szpakowicz
			   Institute of Informatics, Warsaw University   *)

label 1, 2;  (* error halt & almost-fatal error recovery only *)


const
	maxint       =  32767;
	eos             =       0 ;     (* end-of-string character ord *)
	eol             =       10;     (* end-of-line character ord *)
	maxchar         =       127;    (* maximum character ord *)

	(*  table limits  *)
        ctlow   =  0;   cthigh  =  4000;        (* character table *)
        atsize  =  1009;                        (* atom table size: a prime *)
        atlow   =  2;   athigh  =  atlow-1+atsize;        (* *****)
        ttlow   =  0;   tthigh  =  800;         (* trail table *)
        btlow   =  0;   bthigh  =  300;         (* backtrack point table *)
        btbelow =  btlow-1;                     (* for initialisation *)
        ftlow   =  0;   fthigh  =  800;         (* frame table *)
        ftbelow =  ftlow-1;                     (* for initialisation *)

        mtlow   =  2;   mthigh  =  20000;                (* main table *)

	(*  at  *)
	noarity =  -1;          (* used when an atom is only a name handle *)
	emptyatom       =  cthigh;    (*name value marking unused atom entries
					can't be a name - no room for eos *)

        (*  mt  *)
        stacksize  =  5000;     (* size of var & copy stack area *)

        (* mt address constants *)
        stackhigh   = mtlow-1+stacksize;  (* top of var & copy stack area *)
        protlow     = stackhigh+1;        (*  bottom of prototype area *)
        dumvarx     = protlow;            (* the unique dummy variable prototype *)
        nilprotx    = dumvarx+1;          (* a prototype of '[]'/0 *)
        minchprotx  = nilprotx+1;         (* low end of unique char prot area *)
        maxchprotx  = minchprotx+maxchar; (* high end of char prot area *)
        stdintprotx = maxchprotx+1;       (* low end of std int prot area *)

        int        =  1;        (*marker for integer objects, = varlim+1*****)
	varlim     =  0;        (* fields holding  <= varlim are vars,
				   only dumvarx holds varlim            *)
	freevar    =  -1;       (* filler for free vars in stack area *)
	offoff     =  2;        (* variable prot with offset k is
				   represented as  - ( k + offoff )     *)
	minstdint =  -1;        (* integers in this range are given       *)
	maxstdint =  10;        (*     standard prototypes to save memory *)

	(*  various  *)
	maxvarnumb  =  mthigh;          (* max number of different vars
						in a clause - 1         *)
	maxargnumb  =  mthigh;          (* max number of args in a term *)
	maxsyspars =  5;                (* max number of parameters in a
						system routine          *)
	car     =  1;           (* dot's argument number ( sequence head ) *)
	cdr     =  2;           (* _______________ ,, _____________ tail   *)

	(* character class encodings, see array cc & initvars *)
	(*note that <= csmalletter is letter and <= cunderscore is alphameric*)
	cbigletter = 'a';   csmalletter = 'b';   cdigit = 'c';
	cunderscore = 'd';   cbracket = 'e';   csolochar = 'f';
	csymch = 'g';   cother = 'h';

	(* i / o *)
	maxlinesize = 80;


type
	(*  index ranges  *)
	ctx     =  ctlow .. cthigh;     (* character table index *)
	atx     =  atlow .. athigh;     (* atom table index *)
	ttx     =  ttlow .. tthigh;     (* trail table index *)
	btx     =  btlow .. bthigh;     (* backtrack point table index *)
	ftx     =  ftlow .. fthigh;     (* frame table index *)
	ftblwx  =  ftbelow..fthigh;	(* specially for trooverlay *)
	mtx     =  mtlow .. mthigh;     (* main table index *)

	stackx  =  mtx; (* i.e.mtlow .. stackhigh; stack area index in mt *)
	protx   =  mtx; (* i.e.protlow .. mthigh;  prototype area index in mt*)

	(*  sundry ranges  *)
	varnumb  =  0 .. maxvarnumb;    (* number of vars / var offset *)
	argnumb  =  noarity .. maxargnumb;      (* number of arguments *)
	nsysparam =  0 .. maxsyspars;   (* number of system routine params *)
	classofchar     =   cbigletter..cother;

	(* standard atoms ( indexing table std ) :
			';'/2, ','/2, 'call'/1, 'tag'/1,
			'[]'/0, '.'/2, 'error'/1, 'user'/0   *)
	stdatomid  =  ( atmsemcol,  atmcomma,  atmcall,  atmtag,
			atmnil,  atmdot,  atmerror,  atmuser  );


	(*  system routine identifiers  *)
	sysroutid  =  ( idfail,  idtag,  idcall,
			idslash,
			idtagcut,   idtagfail,   idtagexit,  idancestor,
			idhalt,   idstatus,
			iddisplay,  idrch,  idlastch,  idskipbl,  idwch,
			idecho,  idnoecho,  idsee,  idseeing,  idseen,
			idtell,  idtelling,  idtold,
			idordchr,  idsum,  idprod,  idinf,  idnameinf,
			idsmalletter,  idbigletter,  idletter,  iddigit,
			idalphanum,  idbracket,  idsolochar,  idsymch,
			ideqvar,  idvar,  idatom,  idinteger,  idnonvarint,
			idfunctor,  idarg,  idpname,  idpnamei,
			idproc,  idproclimit,   idprocinit,
			idclause,  idretract,  idabolish,
			idassert,  idredefine,  idpredefined,  idprotect,
			idnonexistent,  idnononexistent,
			iddebug,  idnodebug,
			idtrue                  );

	(*  error codes  *)
	errid      =  ( ctovflw,  atovflw,  ttovflw,  btovflw,
			ftovflw,  stackovflw,  protovflw,
			syntax,  loadfile,  sysinit,   usereof,
			longfilename,   unifyerr    );



	(*  objects  *)

	procptr  =  ^ proc;

	proc  =         record          (* a procedure descriptor *)
			  next   :  procptr;    (* what to do after failure *)
			  nvars  :  varnumb;    (* size of variable frame *)
			  case systemroutine : boolean of
			    true  : ( sysid : sysroutid );
			    false : ( head, body : protx )
			end;

	atom  =         record  (* unique functor/predicate representation *)
			  name   :  ctx;        (* printname string,
						   = emptyatom if not used *)
			  arity  :  argnumb;
			  proc   :  procptr     (* associated proc or nil *)
			end;

	backpt  =       record          (* backtrack-point (fail-point) *)
			  frozenframe  :  ftx;          (* where to resume *)
			  resume       :  procptr;      (* what to resume *)
			  copylev      :  stackx;       (* copy stack level *)
			  traillev     :  ttx           (* trail stack level*)
			end;

	frame   =       record          (* environment frame *)
			  anc    :  ftx;        (*to caller frame (but TRO)*)
			  calls  :  mtx;        (* call is 1st in this seq. *)
			  vars   :  stackx      (* variables of this frame *)
			end;


var
        (*** variables ***)

	mc, si, so  : text; (*sysfile,seefile,tellfile*)
	seeing, telling :  boolean;     (* terminal/file i/o flags *)
	seeobj, tellobj :  mtx;         (* terms with current file names *)
	sysloading  :  boolean;         (* on when loading system file *)
	colinesize, solinesize : 0 .. maxlinesize;
	echo     : boolean;     (* input echo switch *)

	ct      :  array [ ctx ] of char;       (* character string table *)
	ctop ,                          (* first free *)
	builtname  :  ctx;              (* first char of constructed name *)

	at      :  array [ atx ] of atom;       (* atom table : hashed *)
	freeatoms : integer;                    (* >= 1, 1 kept as sentinel *)

	tt      :  array [ ttx ] of stackx;     (* trail table *)
	ttop    :  ttx;                         (* first free *)

	bt      :  array [ btx ] of backpt;     (* bactrack-point table *)
	topb    :  btbelow .. bthigh;   (* last used = current backpt *)

	ft      :  array [ ftx ] of frame;      (* environment frame table *)
	topf    :  ftblwx;   (* last used = current frame *)

	mt      :  array [ mtx ] of integer;    (* main memory table *)
	vtop ,                          (* first free above variable stack *)
	csbot ,                         (* first free below term copy stack *)
	gtop ,                  (* first free above ground prot area *)
	ngbot   :  mtx;         (* first free below non-ground prot area *)
	gprotected, ngprotected : protx;  (* clauses with heads below or above
					     these can't be redefined         *)

	(* storage utilisation :  low level markers, for status only *)
	tminfree ,   bminfree ,   fminfree ,
	stackminfree                            :  integer ;

	(* control state registers ( see also topf and topb ) *)
	ancf    :  ftblwx;      (* = ft [ topf ]. anc *)
	topenv ,                (* = ft [ topf ]. vars, current clause's
						variable environment    *)
	ancenv :  stackx;       (* = ft [ ancf ]. vars, current call's
						variable environment    *)
	ccall    :  mtx;        (* the current call *)
	cproc    :  procptr;    (* the current proc *)

	frozenvars ,            (* = ft [ bt [ topb ]. frozenframe ]. vars *)
	frozenheap  :  stackx;  (* = bt [ topb ]. copylev *)

	debug    :  boolean;    (* wallpaper trace switch *)

	(* standard system routine interface *)
	syserror  :  boolean;   (* turned on for bad system routine calls *)
	spar  :  array [ 1 .. maxsyspars ] of mtx; (*parameters (in ancenv)*)
	sparv :  array [ 1 .. maxsyspars ] of integer; (*values of int spars*)

	(* standard atom pointers ( constants ) *)
	std  :  array [ stdatomid ] of atx;

	(* pointers to standard objects in prototype area ( constants) *)
	errorcallseq    :  protx;               (* '.'('error'(:0), '[]') *)
	userprot        :  protx;               (* prototype of 'user'/0 *)

	(* assert etc. *)
	lastasserted    : atx;  (*head atom of most recently asserted clause*)
	redefining      : boolean;      (* redefinition switch, cf.redefine *)
	vardictbot      : stackx;       (* first location of temporary
					   variable dictionary above
					   topmost variable frame - used
					   only by storeclause & helpers  *)
	procindx        : integer;      (* used by genproc & helpers *)
	shownonexistent : boolean;      (* show calls to empty procs ? *)

	(* reader & high i/o *)
	cch      : char;        (* current input character *)
	nclvars  : varnumb;     (* number of vars in currently read clause *)
	whaterr  : errid;       (* syntax or loadfile according to context *)

	(* character class mapping (chars in the external form of a Prolog
	   program, but bigletter through cdigits used also by "internal"
	   form reader). the classes are encoded as characters, as this
	   is the only way to obtain a packed table with some Pascal
	   implementations.                                                *)
	cc      :       array [ char ] of classofchar;

	(* main program variables *)
	terminate  :   boolean;         (* flag set by system routine halt *)
	goalstmnt  :  protx;            (* current initial goal statement *)
	ngoalvars  :  varnumb;          (* and the number of its variables *)



(*---------  e r r o r s   ---------*)

procedure putline;
forward; (*-----*)

procedure halt;
(* this might be implementation-dependent *)
begin   writeln;   writeln ( ' ******toyprolog aborted******');
	goto 1
end;

procedure status;
const field = 8;
type tablename = packed array [ 1..9 ] of char;
	procedure line ( caption : tablename;  size, free, unused : integer );
	begin writeln ( caption, size : field, free : field, unused : field )
	end;
begin   writeln;
	writeln ( ' Toy Prolog   version 1 (MERA)  -  IIUW Warszawa 1983' );
        writeln ( '  table      size    free  unused' );
	line ( 'character', cthigh, cthigh - ctop, cthigh - ctop );
	line ( 'atom     ', atsize, freeatoms, freeatoms );
	line ( 'prototype', mthigh - protlow, ngbot - gtop, ngbot - gtop );
	line ( 'trail    ', tthigh, tthigh - ttop, tminfree );
	line ( 'backpt   ', bthigh, bthigh - topb, bminfree );
	line ( 'frame    ', fthigh, fthigh - topf, fminfree );
	line ( 'var/copy ', stackhigh - mtlow, csbot - vtop, stackminfree );
	colinesize:= 0
end;

procedure errror ( id : errid );
begin   writeln;
	write ( ' ++++++error : ' );
	case id of
	  ctovflw       :  write ( 'character' );
	  atovflw       :  write ( 'atom' );
	  ttovflw       :  write ( 'trail' );
	  btovflw       :  write ( 'backtrack-point' );
	  ftovflw       :  write ( 'frame' );
	  stackovflw    :  write ( 'variable/copy' );
	  protovflw     :  write ( 'prototype' );
	  syntax        :  write ( 'bad system program syntax' );
	  loadfile      :  write ( 'bad system kernel file' );
	  sysinit       :  write ( 'interpreter initialisation error' );
	  usereof       :  write ( 'user eof ' );
	  longfilename  :  write ( 'file name too long' );
	  unifyerr      :  write ( 'error in unify, interpreter bad' )
	end;
	if id <= protovflw then begin
		write ( ' table overflow' );
		putline;   status
	end;
	putline;
	if id in [ ctovflw, protovflw, loadfile, sysinit, usereof ] then halt
	else goto 2
end;




(*---------  l o w   i n p u t / o u t p u t  ---------*)

	(* file opening/closing is implementation-dependent.
	   see also procedure loadsyskernel ( initialisation ). *)

procedure closefile ( forinput : boolean );
(* close mc,si or so, switching to user terminal.
   forinput signals whether the current input or the current output file
   is to be closed. no effect if the file is not open.                   *)
begin   if forinput then begin
		if seeing then begin
			seeing:= false;   seeobj := userprot
		end
		else
		if sysloading then begin
			sysloading:= false;   seeobj := userprot
		end
	end
	else    if telling then begin
			telling:= false;   tellobj := userprot
		end
end (*closefile*);

procedure openfile ( name : ctx;  forinput : boolean );
(* open the file (si or so according to 2nd parameter)
   whose name is given by a string in character table.
   a file is opened rewound. if a previous file is open, it is closed.  *)
const  ln = 35; (* for RSX-11 *)
var   nm : packed array [ 1..ln ] of char;   k : 1..ln;
begin   k:= 1;
	while ( k <> ln ) and ( ct [ name ] <> chr ( eos ) ) do begin
		nm [ k ] :=  ct [ name ];   name:= name + 1;   k:= k + 1
	end;
	if ct [ name ] <> chr ( eos ) then errror ( longfilename );
        for k:= k to ln do nm [ k ] := ' ';
        closefile ( forinput );               (* only 1 file per stream *)
        if forinput then begin
                assign(si, nm);
                reset ( si );    seeing:= true
	end
	else begin
                assign(so, nm);
                rewrite ( so );    telling:= true;
		solinesize:= 0
	end
end (*openfile*);

procedure putline;
(* start a new output line *)
begin   if telling then begin
		 writeln ( so );   solinesize:= 0
	end
	else begin
		writeln;  colinesize:= 0
	end
end;

procedure getline;
(* start a new input line *)
var   keep : boolean;
begin   if sysloading then readln ( mc )
	else
	if seeing then readln ( si )
	else readln;
	if echo then begin
		 keep:= telling;   telling:= false;
		 putline;    telling:= keep
	end
end;

procedure putch ( c : char );
(* put a character onto the current output file *)
(* characters are counted to avoid line overflow *)
begin   if telling then begin
		if solinesize >= maxlinesize then putline;
		write( so, c );   solinesize:= solinesize +1
	end
	else begin
		if colinesize >= maxlinesize then putline;
		write( c );   colinesize:= colinesize + 1
	end
end;

procedure getch ( var c : char );
(* get a character from the current input file *)
var   keep : boolean;
begin   if sysloading then read ( mc, c )
	else
	if seeing then read ( si, c )
	else read ( c );
	if echo then begin
		keep:= telling;   telling:= false;
		putch ( c );   telling:= keep
	end
end;

procedure putint ( i : integer );
(* output an integer without blanks *)
var   divisor : integer;
begin   if i < 0 then begin
		putch ( '-' );   i:= - i
	end;
	divisor:= 10000;                        (***16 bit version***)
	while ( i div divisor = 0 ) and ( divisor <> 1 ) do
						divisor:= divisor div 10;
	repeat  putch ( chr ( i div divisor + ord ( '0' ) ) );
		i:= i mod divisor;   divisor:= divisor div 10
	until divisor = 0
end;

function endofline  : boolean;
(* eoln on the current input file ? *)
begin   if sysloading then endofline:= eoln ( mc )
	else
	if seeing then endofline:= eoln ( si )
	else endofline:= eoln
end;

function endoffile  : boolean;
(* eof on the current input file ? *)
begin   if sysloading then endoffile:= eof ( mc )
	else
	if seeing then endoffile:= eof ( si )
	else endoffile:= eof
end;





(*---------  d i c t i o n a r y  ---------*)

	(* string representation is known to initprot, gterm & nameexplode *)


function compstrings ( s1, s2 : ctx;  equality : boolean )  : boolean;
(* equality --> s1 = s2 ?  #  not equality --> s1 < s2 ?  *)
begin   while ( ct[s1] =  ct[s2])  and  ( ct[s2] <>  chr(eos) ) do begin
		s1:= s1 + 1;   s2:= s2 + 1
	end;
	if equality then compstrings:= ct[s1] = ct[s2]  (* both are eos ? *)
	else compstrings:= ct[s1] < ct[s2]      (* note that eos = 0 *)
end;

function charlast ( string : ctx )  : ctx;
(* locate the last character (except eos) of this string *)
begin   while  ct [ string ] <> chr ( eos )  do string:= string + 1;
	charlast:= string - 1     (*correct because lowest string not empty*)
end;

procedure newatom ( where : atx;  printname : ctx;  narity : argnumb );
(* create a new atom ( at least one empty atom needed as sentinel ) *)
begin   if freeatoms = 1 then errror ( atovflw );
	with at [ where ] do begin
		name:= printname;   arity:= narity;   proc:= nil
	end;
	freeatoms:= freeatoms - 1
end;

function findname ( firstch, lastch : ctx )  : atx;
(* find the first occurence of an atom with name in ct firstch
   through lastch. create one ( with noarity ) if none .        *)
var   a : atx;
begin   (* a simple hash on first and last char *)
	(* ***linear rehash used : don't forget findatom when improving*** *)
	a:= ( ord(ct[firstch])*256 + ord(ct[lastch]) ) mod atsize  + atlow;
	(* rehash loop stopped by empty sentinel ( freeatoms >= 1 ) *)
	while ( at [ a ]. name <> emptyatom ) and
			not compstrings ( at [ a ]. name, firstch, true )  do
		if a <> athigh then a:= a + 1
		else a:= atlow;
	if at [ a ]. name = emptyatom then newatom ( a, firstch, noarity );
	findname:= a
end;

function findatom ( nameatom : atx;  narity : argnumb )  : atx;
(* given the first atom with name, find or create one with name and arity *)
var   a : atx;   thename : ctx;
begin   if at [ nameatom ]. arity = noarity then begin (*just a name handle:*)
		at [ nameatom ].arity:= narity;        (* so use it !       *)
		findatom:= nameatom
	end
	else begin      (* linear rehash (there is an empty atom sentinel) *)
		thename:= at [ nameatom ]. name;   a:= nameatom;
		while ( at [ a ]. name <> emptyatom ) and
			 ( (at[a].name<>thename) or (at[a].arity<>narity) ) do
			if a <> athigh then a:= a + 1
			else a:= atlow;
		if at [a]. name = emptyatom then newatom(a, thename, narity);
		findatom:= a
	end
end;

procedure buildname ( c : char );
(* add a character to the currently-constructed name *)
begin   if ctop = cthigh then errror ( ctovflw );
	ct [ ctop ]:= c;   ctop:= ctop + 1
end;

function wrapname  : atx;
(* the constructed name is complete :  return the first atom with this name *)
var   nm : atx;
begin   buildname ( chr ( eos ) );              (* terminate the string *)
	nm:= findname ( builtname, ctop - 2 );
	if at [ nm ]. name <> builtname then    (* copy already exists    *)
		ctop:= builtname                (* so destroy the new one *)
	else builtname:= ctop;                  (* prepare for next name *)
	wrapname:= nm
end;

function samename ( atom : atx;  narity : argnumb )  : atx;
(* return the first atom ( in the appropriate hash chain ) with
   the same name as atom but with arity ( which may be different )      *)
begin   with at [ atom ] do
	samename:= findatom ( findname ( name , charlast ( name ) ), narity )
end;

procedure putname ( c : ctx );
(* output the print name starting at c *)
begin   while ct [ c ] <> chr ( eos ) do begin
		putch ( ct [ c ] );   c:= c + 1
	end
end;





(*---------  t e r m   r e p r e s e n t a t i o n  ---------*)

	(* pt stands for a prototype or a term instance *)
	(* note: chars have unique prots, cf.newprot & charprot,
		 so have integers in the most frequently used range *)


procedure protspace ( size : integer );
(* enough space in prototype area ? *)
begin   if ngbot - gtop < size then errror ( protovflw )
end;

function isvar ( pt : mtx )  : boolean;
(* a variable ? *)
begin   isvar:=  mt [ pt ] <= varlim
end;

function isndvar ( pt : mtx )  : boolean;
(* a non-dummy variable ? *)
(** expanded in unify, terminst & argument *)
begin   isndvar:= mt [ pt ] < varlim
end;

function isint ( pt : mtx )  : boolean;
(* an integer ? *)
(** expanded in unify *)
begin   isint:=  mt [ pt ] = int
end;

function intval ( pt : mtx )  : integer;
(* an integer :  return its value *)
(** expanded in unify *)
begin intval:=  mt [ pt + 1 ]
end;

function is_normal ( pt : mtx )  : boolean;
(* not a variable or integer ?   note that int = varlim + 1 *)
(** expanded in unify *)
begin   is_normal:=  mt [ pt ]  >  int
end;

function isargless ( pt : mtx )  : boolean;
(* a non-variable, non-integer prot/term without arguments ? *)
begin   if mt [ pt ] <= int then isargless:= false
        else isargless :=   at [ mt [ pt ] ] . arity  =  0
end;

function ischar ( pt : mtx;  var c : char )  : boolean;
(* is it a character term ?  if so, what is the character ? *)
var   val : integer;
begin   val:= pt - minchprotx;
        if ( val < 0 ) or ( val > maxchar ) then ischar:= false
        else begin
                c:= chr ( val );   ischar:= true
        end
end;

function charprot ( c : char )  : protx;
(* return this character's prototype *)
begin   charprot:= minchprotx + ord ( c )
end;

function getarity ( pt : mtx )  : argnumb;
(* return the narity of pt's functor *)
(** expanded in unify and terminst *)
begin   getarity:=  at [ mt [ pt ] ] . arity
end;

function getarg ( pt : mtx;  n : argnumb )  : mtx;
(* return pt's n'th argument (for prots only prot arg returned) *)
(** expanded in argument *)
begin   (* check(1, is_normal( pt ) and ( n>= 0 ) and ( n <= getarity( pt ))); *)
        if mt [ pt + n ] <= varlim then getarg:= pt + n
        else getarg:= mt [ pt + n ]
end;

function isstdint ( val : integer;  var prot : protx )  : boolean;
(* if val is a standard integer, return its unique prototype *)
begin   if ( val < minstdint ) or ( val > maxstdint ) then isstdint:= false
        else begin
                prot:= stdintprotx + ( val - minstdint ) * 2;
                isstdint:= true
        end
end;

procedure mkint ( val : integer );
(* build an integer prototype on top of ground area *)
begin   protspace ( 2 );
        mt [ gtop ] := int;   mt [ gtop + 1 ] := val;   gtop:= gtop + 2
end;

function newintprot ( val : integer )  : protx;
(* return an integer prototype (ground) *)
var   p : protx;
begin   if isstdint ( val, p ) then newintprot:= p
        else begin
                newintprot:= gtop;   mkint ( val )
        end
end;

function initprot ( atom : atx )  : protx;
(* create a prototype whose functor is described by atom.
   note that it will need argument filling and wrapping.
   guards uniqueness of character prototypes.            *)
var   size : integer;   achar : boolean;
begin   with at [ atom ] do begin
                size:= 1 + arity;               (* 1 for the functor *)
                if size <> 1 then achar:= false
                else achar:= ct [ name + 1 ] = chr ( eos );
                if achar then initprot:= charprot ( ct [ name ] )
        end;
        if not achar then begin
                protspace ( size );   ngbot:= ngbot - size;
                mt [ ngbot + 1 ] := atom;               (* functor filled *)
                initprot:= ngbot + 1
        end
end;

procedure newpvararg ( p : protx;  n : argnumb;  v : varnumb );
(* insert this variable offset as the n'th argument of a constructed prot *)
begin   mt [ p + n ] :=  - ( v + offoff )
end;

procedure newparg ( p : protx;  n : argnumb;  arg : integer );
(*insert the n'th (non-var or dummy var) argument into constructed prototype*)
begin   (* check (2, arg >= protlow ); *)
        mt [ p + n ] := arg
end;

function wrapprot ( p : protx )  : protx;
(* p is a recently constructed prototype. return its final address
   after attempting to move it to the ground prot area. the manner of
   construction ensures that p can't be ground if it is not directly above
   ngbot ( or already in the ground area - applies to chars & std ints ).   *)
var   arity, k : argnumb;   ground : boolean;
begin   if p <> ngbot + 1 then wrapprot:= p
        else begin                      (* see if all args are ground *)
                arity:= getarity ( p );   k:= 0;   ground:= true;
                while ( k <> arity ) and ground do begin
                        k:= k + 1;   ground :=  getarg ( p, k )  <  gtop
                end;
                if not ground then wrapprot:= p
                else begin                      (* shift to ground area *)
                        wrapprot:= gtop;
                        for k:= 0 to arity do begin     (*0 for the functor*)
                                ngbot:= ngbot + 1;
                                mt [ gtop ] := mt [ ngbot ];
                                gtop:= gtop + 1
                        end
                end (*ground*)
        end (*ground?*)
end (*wrapprot*);





(*---------  t e r m   i n s t a n c e s   ---------*)

        (* pt stands for a copy stack term instance or
           for a prototype in variable environment env *)


procedure stackspace ( size : integer );
var   left : integer;
begin   left:= csbot - ( vtop + size );
        if left < 0 then errror ( stackovflw );
        if left < stackminfree then stackminfree:= left
end;

procedure pushvars ( n : varnumb );
(* push n free variables onto the variable environment stack *)
begin   if n <> 0 then begin    (* test just for efficiency *)
                stackspace ( n );
                for n:= n downto 1 do begin
                        mt [ vtop ] :=  freevar;   vtop:= vtop + 1
                end
        end
end;

function heapint ( i : integer )  : mtx;
(* create an integer object on the copy stack or return std int prot *)
var   p : protx;
begin   if isstdint ( i, p ) then heapint:= p
        else begin
                stackspace ( 2 );
                mt [ csbot ] :=  i;   mt [ csbot - 1 ] :=  int;
                heapint:= csbot - 1;   csbot:= csbot - 2
        end
end;

function heapdot ( head, tail : mtx )  : stackx;
(* create an instance of  .(head, tail)  *)
begin   stackspace ( 3 );
        mt [ csbot ] :=  tail;   mt [ csbot - 1 ] :=  head;
        mt [ csbot - 2 ] :=  std [atmdot];   heapdot:= csbot - 2;
        csbot:= csbot - 3
end;

function gterm ( atom : atx )  : stackx;
(* create a most general term instance for this atom, i.e. a term with
   different variables for arguments, but guard char uniqueness.           *)
var   thisarity : argnumb;   thisname : ctx;
begin   with at [ atom ] do begin
                thisarity:= arity;   thisname:= name
        end;
        if ( thisarity = 0 ) and ( ct [ thisname + 1 ] = chr( eos ) ) then
                                        gterm:= charprot ( ct [ thisname ] )
        else begin
                stackspace ( 1 + thisarity );   (* 1 for the functor *)
                for thisarity:= thisarity downto 1 do begin
                        mt [ csbot ] :=  freevar;   csbot:= csbot - 1
                end;
                mt [ csbot ] :=  atom;   gterm:= csbot;   csbot:= csbot - 1
        end
end;

procedure bindv ( v : stackx;  t : mtx );
(* bind variable instance v to term t, pushing a trail if necessary *)
begin   (* check (3, isndvar ( v ) and ( mt [ v ] = freevar ) ); *)
        mt [ v ]:=  - t;
        if ( v > frozenheap ) or ( v < frozenvars ) then begin
                if ttop = tthigh then errror ( ttovflw );
                tt [ ttop ] := v;   ttop:= ttop + 1;
                if tminfree > tthigh - ttop then tminfree:= tthigh - ttop
        end
end;

procedure undo ( tlev : ttx );
(* bring trail level down to tlev and undo the bindings referenced by
   popped trail entries.                                                *)
begin   while ttop > tlev do begin
                ttop:= ttop - 1;   mt [ tt [ ttop ] ] :=  freevar
        end
end;

procedure bind2vars ( v1, v2 : stackx );
(* v1 and v2 are free variable instances. if they are of the same kind,
   bind the younger to the older, otherwise bind the frame variable to
   the copy stack ( heap ) variable.
   ** v < vtop is expanded isframevar                                   *)
begin   (* check (4, isndvar ( v1 ) and isndvar ( v2 ) ); *)
        if v1 = v2 then         (* no action - must remain free *)
        else
        if v1 < vtop then                                       (*v1 frame*)
                if v2 < vtop then                               (*v2 frame*)
                        if v1 < v2 then bindv ( v2, v1 )         (*v1 older*)
                        else bindv ( v1, v2 )                    (*v2 older*)
                else bindv ( v1, v2)                     (*v1 frame, v2 heap*)
        else
        if v2 < vtop then bindv ( v2, v1 )               (*v2 frame, v1 heap*)
        else
        if v1 < v2 then bindv ( v1, v2 )                         (*v2 older*)
        else bindv ( v2, v1 )                                    (*v1 older*)
end;

function varadr ( v : mtx;  env : stackx )  : stackx;
(* v is a variable prototype or instance. if a prototype,
   find its instance in variable environment env. **isnongroundprot expanded *)
begin   (* check (5, isndvar ( v ) ); *)
        if not ( v > ngbot ) then varadr:= v
        else varadr:= env + ( - mt [ v ] - offoff )
end;

function argument ( pt : mtx;  env : stackx;  n : argnumb )  : mtx;
(* pt is a term instance or a prototype in variable environment env,
   and is neither a variable nor an integer. return its dereferenced
   n'th argument. instead of a variable prototype return
   a frame variable cell index. ** getarg expanded here.                *)
var   field, arg : mtx;
begin   field:= pt + n;
        if mt [field] > varlim then arg:= mt [field]            (* non-var *)
        (* mt [ field ] = varlim impossible, as only dumvarx holds varlim *)
        else begin      (* non-dummy var, dereference binding if any *)
                arg:= varadr ( field, env );
                (*note that nonvars or free vars will stop the loop *)
                while mt [ arg ] < freevar do arg:= - mt [ arg ]
        end;
        argument:= arg
end;

function terminst ( pt : mtx;  env : stackx )  : mtx;
(* if pt is a term instance ( a ground prototype or a copy-stack object ),
   return its address. if it is a non-ground prototype use env as
   the proper variable environment and return the address of a new copy
   in the copy stack. free frame variables are bound to their copies.     *)
(* ***isnongroundprot, getarity and isndvar expanded *)
var   arity, k : argnumb;   copy : stackx;   arg : mtx;
begin   if not ( pt > ngbot ) then terminst:= pt     (* not isnongroundprot *)
        else    (* the following branch for stand-alone frame variables *)
        if mt[ pt ] < varlim then begin  (*isndvar: build a new heap variable*)
                stackspace ( 1 );
                mt [ csbot ] := freevar;   copy:= csbot;   csbot:= csbot - 1;
                bindv ( varadr ( pt, env ), copy );   terminst:= copy
        end
        else begin
                arity:= at [ mt [ pt ] ] .arity;
                stackspace ( 1 + arity );       (* 1 for the functor *)
                csbot:= csbot - ( 1 + arity );   copy:= csbot + 1;
                mt [ copy ] := mt [ pt ];               (* functor *)
                for k:= 1 to arity do begin             (* arguments *)
                        arg:= argument ( pt, env, k );
                        if arg < vtop then begin  (*framevar,evidently free*)
                                bindv ( arg, copy + k );
                                mt [ copy + k ] :=  freevar
                        end
                        else mt [ copy + k ] := - terminst ( arg, env )
                             (*ie.args always linked as if through vars,
                               because terminst can return a variable.  *)
                end;
                terminst:= copy
        end
end;

function unify ( c : mtx;  cenv : stackx;  h : mtx;  henv : stackx )
                                                                : boolean;
(* c and h are terms ( possibly prototypes in variable environments cenv and
   henv ), but neither is a non-dummy variable prototype ( cf. argument ).
   try to unify, but don't undo on failure.                                 *)
var   success : boolean;   arity, n : argnumb;   mth, mtc : integer;
begin   (* check(6, not ( isndvar( c ) and isnongroundprot( c ) ) and
                        not ( isndvar( h ) and isnongroundprot( h ) ) ); *)
        success:= true;         (* let's be optimistic for a change *)
        mth:= mt [ h ];   mtc:= mt [ c ];       (* more wieldy & efficient *)
        (* first branch if same prots and environment or same terms and
           environment doesn't matter (i.e. not non-ground prots),
           i.e. an attempt to unify a term with itself : no action required *)
        if ( h = c ) and ( ( henv = cenv ) or ( h <= ngbot ) )  then
        else
        if ( mth > int ) and ( mtc > int ) then         (*both are "normal" *)
                if mth <> mtc then success:= false      (*different functors*)
                else begin                              (*try the arguments *)
                        n:= 0;   arity:= at [ mth ]. arity;
                        while ( n <> arity ) and success do begin
                                n:= n + 1;
                                success:= unify( argument( c, cenv, n ), cenv,
                                                 argument( h, henv, n ), henv)
                        end
                end
        else                                    (* at least one int or var *)
        if ( h = dumvarx ) or ( c = dumvarx ) then    (*no action*)
        else
        if mth = freevar then                                    (*isndvar(h)*)
                if mtc = freevar then bind2vars ( h, c )         (*isndvar(c)*)
                else bindv ( h, terminst ( c, cenv ) )
        else
        if mtc = freevar then bindv ( c, terminst ( h, henv ) )   (*isndvar(c)*)
        else
        if mth = int then                                       (*isint(h)*)
                if mtc <> int then success:= false      (* not isint( c ) *)
                else success:= mt [ h + 1 ] = mt [ c + 1 ]      (*intvals*)
        else
        if mtc = int then success:= false
        else errror ( unifyerr );
        unify:= success
end;

procedure display ( pt : mtx;  env : stackx );
(* output pt in standard form *)
var   arity, k : argnumb;
begin   if isvar ( pt ) then begin
                putch ( '_' );
                if pt <> dumvarx then putint ( varadr ( pt, env ) )
        end
        else
        if isint ( pt ) then putint ( intval ( pt ) )
        else begin
                putname ( at[ mt[ pt ] ].name );   arity:= getarity ( pt );
                if arity <> 0 then begin
                        putch ( '(' );   putch ( ' ' );
                        for k:= 1 to arity do begin
                                display ( argument ( pt, env, k ), env );
                                if k <> arity then putch ( ',' );
                                putch ( ' ' )
                        end;
                        putch ( ')' )
                end (*arity<>0*)
        end
end (*display*);

function iscallsequence ( pt : mtx;  env : stackx ) : boolean;
(* is this term a dotted list without integer or variable elements ? *)
var   is : ( no, dontknow, yes );
begin   is:= dontknow;
        repeat  if not ( is_normal ( pt ) ) then is:= no
                else
                if mt [ pt ] = std [atmnil] then is:= yes
                else
                if mt [ pt ] <> std [atmdot] then is:= no
                else begin
                        if not is_normal( argument(pt, env, car)) then is:= no;
                        pt:= argument ( pt, env, cdr )          (* tail *)
                end
        until is <> dontknow;
        iscallsequence:=  is = yes
end;





(*---------  c o n t r o l  ---------*)

        (* note that call sequence prototypes have non-variable elements.
           thus mt [ seq + car ] and mt [ seq + cdr ] access the first
           call in a sequence and the sequence tail respectively.         *)


procedure sysroutcall ( id : sysroutid;  var success, stop : boolean );
forward; (*----------------------------------------------------------*)


function isnonunitclause ( proc : procptr )  : boolean;
(* is proc a clause, and does it have any calls ? *)
begin   if proc = nil then isnonunitclause:= false
        else with proc ^  do
                if systemroutine then isnonunitclause:= false
                else isnonunitclause:= mt [ body ] <> std [atmnil]
end;

procedure pushbackpt ( proc : procptr );
(* push a backtrack point freezing the current frame for resumption of proc *)
begin   if topb = bthigh then errror ( btovflw );
        topb:= topb + 1;
        if bminfree > bthigh - topb then bminfree:= bthigh - topb;
        with bt [ topb ] do begin
                frozenframe:= topf;   resume:= proc;
                copylev:= csbot;   traillev:= ttop
        end;
        frozenvars:= topenv;   frozenheap:= csbot
end;

procedure popbackpt;
(* topb <> btlow. return to the previous backtrack point *)
begin   (* check (7, topb <> btlow ); *)
        topb:= topb - 1;
        with bt [ topb ] do begin
                frozenheap:= copylev;   frozenvars:= ft [ frozenframe ]. vars
        end
end;

procedure pushframe ( ancestor : ftx;  callsequence : protx );
(* create a new (current) environment frame with no variables (as yet) *)
begin   if topf = fthigh then errror ( ftovflw );
        topf:= topf + 1;
        if fminfree > fthigh - topf then fminfree:= fthigh - topf;
        with ft [ topf ] do begin
                anc:= ancestor;   calls:= callsequence;   vars:= vtop
        end;
        topenv:= vtop;   ancf:= ancestor;   ancenv:= ft [ ancf ]. vars
end;

procedure backtrack ( var stop : boolean );
(* set stop if no backtrack points left; otherwise restore the state frozen
   by the current backpt and set cproc to the clause to be attempted next   *)
begin   if topb = btlow then stop:= true
        else begin
                csbot:= frozenheap;   vtop:= frozenvars;
                with bt [ topb ] do begin
                        undo ( traillev );
                        (* if frozenframe = topf, we are only backtracking
                           from a non-matching clause head : the control
                           registers need not change.                      *)
                        if frozenframe <> topf then begin
                                topf:= frozenframe;   topenv:= ft [topf].vars;
                                ancf:= ft [topf].anc; ancenv:= ft [ancf].vars;
                                (* get ccall, skipping call & tag :
                                   no error possible, we've been here *)
                                ccall:= mt [ ft [ topf ]. calls + car ];
                                while ( mt [ ccall ] = std [atmcall] ) or
                                      ( mt [ ccall ] = std [atmtag] ) do
                                        ccall:= argument( ccall, ancenv, 1 )
                        end;
                        cproc:= resume ^. next;
                        if cproc = nil then popbackpt
                        else
                        if cproc ^. next = nil then popbackpt
                        else resume:= cproc
                end (*with*)
        end
end (*backtrack*);

procedure exitt ( var stop : boolean );
(* starting from the most recently activated procedure ( system routine or
   unit clause ) exit until a pending call is found. set stop if none.     *)
var   f : ftx;   nextcseq : protx;   found : boolean;
begin   found:= false;   f:= topf;   stop:= false;
        repeat  (* tentative exit until nonterminated frame or stack bottom *)
                nextcseq:= mt [ ft [ f ]. calls + cdr ];
                if mt [ nextcseq ] = std [atmdot] then found:= true
                else begin                              (* no more calls *)
                        f:= ft [ f ]. anc;   stop :=  f = ftlow
                end
        until found or stop;
        if found then begin     (* finalise exit and get next proc *)
                ancf:= ft [ f ]. anc;   ancenv:= ft [ ancf ]. vars;
                if f > bt [ topb ]. frozenframe then topf:= f
                else begin              (* frozen : must activate a copy *)
                        topf:= bt [ topb ]. frozenframe + 1;
                        with ft [ topf ] do begin
                                anc:= ancf;   vars:= frozenvars
                        end
                end;
                with ft [ topf ] do begin       (* advance in frame *)
                        calls:= nextcseq;   topenv:= vars
                end;
                vtop:= topenv   (* discard exited nonfrozen var env frames *)
        end
end (*exitt*);

function candotro ( frame : ftx )  : boolean;
(* can we apply tail recursion optimisation (tro) to this frame, i.e.
   destroy it after shifting its variables over those of its ancestor ?
   we can if its call is last in its clause ( and is not an outermost
   tag ) and the ancestor is not frozen.                                *)
begin   with ft [ frame ] do
                if anc <= bt [ topb ]. frozenframe then candotro:= false
                else candotro:= ( mt [ mt [calls + cdr] ] = std [atmnil] ) and
                                ( mt [ mt [ calls + car ] ] <> std [atmtag] )
end;

procedure trooverlay ( var youngframe : ftblwx;  var youngenv : stackx;
                                var oldframe : ftblwx;  var oldenv : stackx );
(* apply tail recursion optimisation :  overlay old frame's variable
   environment (oldenv) with that of youngframe (youngenv), then
   let youngframe reference oldframe and oldframe - oldframe's ancestor.
   youngenv must be the youngest variable environment on the stack.
   both frames must not be frozen (vars  not referenced from trail)     *)
var   low, high : stackx;   shiftby, v : integer;
begin   if oldenv <> youngenv then begin  (*shift after reference adjustment*)
                high:= youngenv;   shiftby:= youngenv - oldenv;
                while high < vtop do begin  (*adj. refs to shifted/overlaid*)
                        v:=  - mt [ high ];     (* value of shifted var *)
                        if ( v >= oldenv ) and ( v < youngenv ) then begin
                                (*ref to overlaid. if overlaid free, reverse
                                  pointers so that all vars bound to it will
                                  be rebound to high; otherwise follow the
                                  overlaid ref & repeat adjustment loop for
                                  this high to deal with new mt[high].      *)
                                if mt [ v ] = freevar then begin
                                        mt [ high ] :=  freevar;
                                        mt [ v ] :=  - high;
                                        high:= high + 1
                                end
                                else mt [ high ] := mt [ v ]
                        end (*to overlaid*)
                        else begin   (*if ref. to shifted, adjust to new adr*)
                                if ( v >= youngenv ) and ( v < vtop ) then
                                        mt [ high ] :=  - ( v - shiftby );
                                high:= high + 1
                        end
                end (*adjustment loop*);
                low:= oldenv;
                for high:= youngenv to vtop - 1 do begin        (* shift *)
                        mt [ low ] :=  mt [ high ];   low:= low + 1
                end;
                vtop:= low
        end (*tricky dealings*);
        youngenv:= oldenv;   youngframe:= oldframe;
        oldframe:= ft [ youngframe ]. anc;   oldenv:= ft [ oldframe ]. vars
end (*trooverlay*);

procedure enter;
(* build a frame for the first call in cproc *)
begin   (* check (8, ( cproc <> nil ) and not cproc^.systemroutine ); *)
        if candotro ( topf ) then trooverlay ( topf, topenv, ancf, ancenv );
        pushframe ( topf, cproc ^. body )
end;

procedure errsetup;
(* invoked upon detection of an erroneous system routine call ( so topf is not
   frozen ! ). top frame is associated with a variable bound to the bad call,
   then  a call to 'error'/1 is set up.                                     *)
begin   (* check (9, topf > bt [ topb ]. frozenframe ); *)
        topenv:= vtop;   ft [ topf ]. vars:= topenv;   pushvars ( 1 );
        bindv ( topenv,   terminst ( mt [ ft [topf].calls + car ], ancenv ) );
        pushframe ( topf, errorcallseq );   ccall:= mt [ errorcallseq + car ]
end;

procedure preparestep;
(* load ccall, stripping off tag & call, then load cproc and
   push a backtrack-point if needed.                            *)
begin   ccall:= mt [ ft [ topf ]. calls + car ];
        while ( mt [ ccall ] = std [atmcall] ) or
              ( mt [ ccall ] = std [atmtag] ) do begin
                ccall:= argument ( ccall, ancenv, 1 );
                if not is_normal ( ccall ) then errsetup      (*affects ccall*)
        end;
        cproc:= at [ mt [ ccall ] ]. proc;
        if cproc <> nil then
                if cproc ^. next <> nil then pushbackpt ( cproc )
end (*preparestep*);

procedure onestep ( var success, stop : boolean );
(* perform a single resolution step.
   on entry success is false immediately after a backtrack, true otherwise. *)
        procedure showcall;
        (* show the call in wallpaper trace *)
        var  keep : boolean;
        begin   keep:= telling;   telling:= false;
                display ( ccall, ancenv );  telling:= keep
        end;
        procedure showresult;
        (* show the call's result in wallpaper trace *)
        var  keep : boolean;
        begin   keep:= telling;   telling:= false;
                if success then writeln ( '*+*' )
                else writeln ( '*-*' );
                putline;   telling:= keep
        end;
        procedure shownoproc;
        (* output a message for a call to a nonexistent procedure *)
        var  keep : boolean;
        begin   keep:= telling;   telling:= false;
                putline;
                write ('---failing call to non existent procedure:');
                putline;   putname ( at[ mt[ ccall ] ].name );
                putch ( '/' );
                putint( getarity ( ccall ) );   putline;
                telling:= keep
        end;

begin   if success then preparestep;    (* otherwise backtrack did *)
        if debug then showcall;
        if cproc = nil then begin             (*empty proc called*)
                success:= false;
                if shownonexistent then shownoproc
        end
        else with cproc ^  do begin
                pushvars ( nvars );
                if systemroutine then sysroutcall ( sysid, success, stop )
                else success:= unify( ccall, ancenv, cproc ^. head, topenv )
        end (*with*);
        if debug then showresult
end (*onestep*);

procedure startresolution ( goalsequence : protx;  ngoalvars : varnumb );
(* initialise stacks in preparation for this goal's execution. the lowermost
   frame is associated with goal's variables, the lowermost backtrack point
   freezes it and acts as a sentinel. both have some dummy entries.         *)
begin   ttop:= ttlow;   vtop:= mtlow;   csbot:= stackhigh;
        topf:= ftbelow;   topb:= btbelow;
        pushframe ( ftlow, nilprotx );                  (* dummy call *)
        pushbackpt ( nil );
        pushvars ( ngoalvars );   pushframe ( topf, goalsequence )
end;

procedure resolve ( goalsequence : protx;  nvarsingoalseq : varnumb );
(* execute this ( initial ) goal sequence *)
var   success, stop : boolean;
begin   startresolution ( goalsequence, nvarsingoalseq );
        stop:= false;   success:= true;  (* false only on failure *)
        repeat
                onestep ( success, stop );
                if not stop then
                        if syserror then begin
                                syserror:= false;   success:= true;
                                errsetup
                        end
                        else
                        if not success then backtrack ( stop )
                        else
                        if isnonunitclause ( cproc ) then enter
                        else exitt ( stop )
        until stop
end (*resolve*);




(*---------  c o n t r o l   d i s r u p t i o n   e t c .  ---------*)

        (* note that only explicit, outermost tags are detected.
           eg. in "call( tag(...))"  tag is ignored :
           tags are tricky enough as it is.
           cf. also candotro, backtrack and preparestep in control. *)

procedure purgetrail ( low : ttx );
(* remove unnecessary trail entries at and above low , either after
   a successful unifyordont call or after popping backtrack-points in
   a non-backtracking context ( unfreeze ). this is necessary, as
   unfrozen vars might be moved or destroyed - it also saves trail space. *)
var   high : ttx;
begin   for high:= low to ttop - 1 do
                if (tt [high] < frozenvars) or (tt [high] > frozenheap)
                                                                    then begin
                        tt [ low ] :=  tt [ high ];   low:= low + 1
                end;
        ttop:= low
end;

function unifyordont ( c : mtx; cenv : stackx; h : mtx; henv : stackx )
                                                                    : boolean;
(* same as unify, but undo effects on failure.
   this is done by ensuring that all bindings are trailed. *)
var   success : boolean;   fv, fh : stackx;   tlev : ttx;
begin   fv:= frozenvars;   fh:= frozenheap;   tlev:= ttop;
        frozenvars:= vtop;   frozenheap:= csbot;
        success:= unify ( c, cenv, h, henv );
        frozenvars:= fv;   frozenheap:= fh;
        if success then purgetrail ( tlev )
        else undo ( tlev );
        unifyordont:= success
end;

procedure unfreeze ( frame : ftx );
(* remove as many backtrack points as necessary to unfreeze frame ( >ftlow) *)
var   changed : boolean;
begin   changed:= false;
        while frame <= bt [ topb ]. frozenframe do begin (*sentinel at btlow*)
                changed:= true;   popbackpt
        end;
        if changed then purgetrail ( bt [ topb ]. traillev )
end;

procedure cutcleanup;
(* destroy cut-off frames between cut or tagcut and its immediate ancestor. *)
var   ancancf : ftblwx;   ancancenv : stackx;   owncalls : protx;
begin   if topf <> ancf + 1 then begin
                vtop:= ft [ ancf + 1 ] .vars;
                ft [ ancf + 1 ] :=  ft [ topf ];   topf:= ancf + 1;
                ft [ topf ] .vars := vtop
        end;
        if candotro ( ancf ) then begin         (* tro for ancf *)
                ancancf:= ft [ ancf ]. anc;
                ancancenv:= ft [ ancancf ]. vars;
                owncalls:= ft [ topf ]. calls;   (*save continuation*)
                trooverlay( ancf, ancenv, ancancf, ancancenv );
                topf:= ancf + 1;   (*  args were just changed *)
                with ft [ topf ] do begin       (*rebuild own frame*)
                        calls:= owncalls;   anc:= ancf;
                        vars:= vtop;   topenv:= vtop
                end
        end (*tro*)
end (*cutcleanup*);

procedure slash;
(* perform the "cut" operation :  unfreeze the nearest ancestor which
   is not ';'(_,_) or ','(_,_) (after stripping off "tag" and "call").
   attempt a delayed tro for the immediate ancestor.                    *)
var  f,ancancf : ftx;  ancancenv : stackx;  found : boolean;
     call : mtx;
begin   f:= ancf;   found:= false;
        while ( f <> ftlow ) and not found do begin
                call:= mt [ ft [ f ]. calls + car ];
                ancancf:= ft [ f ]. anc;   ancancenv:= ft [ ancancf ]. vars;
                while (mt[call] = std[atmtag]) or (mt[call] = std[atmcall]) do
                                       call:= argument ( call, ancancenv, 1 );
                if ( mt [ call ] <> std [ atmsemcol ] ) and
                        ( mt [ call ] <> std [ atmcomma ] ) then found:= true
                else f:= ancancf
        end;
        if found then begin
                unfreeze ( f );   cutcleanup
        end (*found*)
end (*slash*);

function gottagged ( var f : ftx;   pt : mtx;   env : stackx )  : boolean;
(* among the ancestors of f search for the nearest tagged call unifiable
   with pt in env. if found, unify and return true with f pointing at
   the tagged frame; if not, return false with f undefined.                *)
var   call : protx;   callenv : stackx;   found : boolean;
begin   found:= false;
        while ( f <> ftlow ) and not found do
                with ft [ f ] do begin
                        call:= mt [ calls + car ];   callenv:= ft [anc]. vars;
                        if mt [ call ] = std [atmtag] then found:=
                                unifyordont( argument(call, callenv, 1),
                                                        callenv, pt, env );
                        if not found then f:= anc
                end (*with*);
        gottagged:= found
end;

procedure ancestor ( var success : boolean );
(* try to unify sysrout arg with a tagged ancestor. *)
var   frame : ftx;
begin   frame:= ancf;   success:= gottagged ( frame, spar [ 1 ], ancenv )
end;

procedure tagcut ( var success : boolean );
(* if a tagged ancestor unifies with the sysrout parameter, unfreeze it *)
var   frame : ftx;
begin   frame:= ancf;
        if gottagged ( frame, spar [ 1 ], ancenv ) then begin
                unfreeze ( frame );   cutcleanup
        end
        else success:= false
end;


procedure tagfail ( var success : boolean );
(* if a tagged ancestor unifies with the sysrout parameter, cause
   the ancestor to fail immediately, otherwise simply fail.                *)
begin   tagcut ( success );
        success:= false         (* if ancestor unfrozen, it will fail now *)
end;

procedure tagexit ( var success : boolean );
(* if a tagged ancestor unifies with the sysrout parameter, unify and
   immediately exit to the ancestor by resetting ancf and changing topf's
   call sequence to errorcallseq ( only because it has an empty tail, sorry ! ),
   and cproc to nil. otherwise fail.                                      *)
var   frame : ftx;
begin   frame:= ancf;
        if gottagged ( frame, spar [ 1 ], ancenv ) then
                with ft [ topf ] do begin
                        calls:= errorcallseq;   cproc:= nil;
                        anc:= frame;   ancf:= frame
                end
        else success:= false
end;




(*---------  a r i t h m e t i c ,  c h a r s  ---------*)

procedure bindpar ( v : nsysparam;  t : mtx );
(* v is the number of a variable sysrout parameter :  bind it to t.
   -----> used also outside arithmetic <-----                           *)
begin   if spar [ v ] <> dumvarx then bindv ( varadr( spar [ v ], ancenv ), t )
end;

procedure arithans ( ansvar : nsysparam;  answer : integer );
(* sysrout parameter ansvar is a variable :  bind it to answer *)
begin   bindpar ( ansvar, heapint ( answer ) )
end;

procedure arithpars ( k : nsysparam;  var code : integer );
(* k is the number of parameters in spar. for each integer parameter set
   a bit in the rightmost k bits of code. only integers and variables
   allowed:  syserror is set for others.                                    *)
var   i : nsysparam;
begin   code:= 0;
        for i:= 1 to k do begin
                code:= code * 2;                (* shift left *)
                if isint ( spar [ i ] ) then code:= code + 1    (* set bit *)
                else
                if not isvar ( spar [ i ] ) then syserror:= true
        end
end (*arithpars*);

procedure sum ( var success : boolean );
(* a + b = x  or  a + x = c  or  x + b = c  or  a + b = c ? *)
var   code : integer (* 0..7 *);
begin   arithpars ( 3, code );
        if not syserror then
                case code of
                0, 1, 2, 4 :  syserror:= true;  (* not enough integers *)
(*011*)         3 :  arithans ( 1, sparv [ 3 ] - sparv [ 2 ] );
(*101*)         5 :  arithans ( 2, sparv [ 3 ] - sparv [ 1 ] );
(*110*)         6 :  arithans ( 3, sparv [ 1 ] + sparv [ 2 ] );
(*111*)         7 :  success :=   sparv [ 1 ] + sparv [ 2 ] = sparv [ 3 ]
                end
end (*sum*);

procedure prod ( var success : boolean );
(* a*b+c = x  or  a*b+x = d  or  a*x+c = d or ( a*x+y = d  &  y < a ) or
   ( x*b+y = d  &  y < a )   or  x*b+c = d  or  a*b+c = d ?              *)
var   code : integer (* 0..15 *);
        procedure divmod ( x, by : nsysparam );
        begin   if sparv [ by ] = 0  then syserror:= true
                else begin
                        arithans ( 3, sparv [ 4 ] mod sparv [ by ] );
                        arithans ( x, sparv [ 4 ] div sparv [ by ] )
                end
        end;
        procedure division ( x, by : nsysparam );
        begin   if sparv [ by ] = 0  then syserror:= true
                else begin
                        if (sparv[ 4 ] - sparv[ 3 ]) mod sparv[ by ] <> 0
                                                        then success:= false
                        else arithans(x, (sparv[4] - sparv[3]) div sparv[by])
                end
        end;
begin   (* prod *)
        arithpars ( 4, code );
        if not syserror then
                case code of
                0,1,2,3,4,6,8,10,12 :  syserror:= true;  (* ambiguous *)
(*0101*)        5  :  divmod ( 1, 2 );
(*0111*)        7  :  division ( 1, 2 );
(*1001*)        9  :  divmod ( 2, 1 );
(*1011*)        11 :  division ( 2, 1 );
(*1101*)        13 :  arithans ( 3, sparv [ 4 ] - sparv [ 1 ] * sparv [ 2 ] );
(*1110*)        14 :  arithans ( 4, sparv [ 1 ] * sparv [ 2 ] + sparv [ 3 ] );
(*1111*)        15 :  success := sparv [1] * sparv [2] + sparv [3] = sparv [4]
                end
end (*prod*);

procedure inf ( var success : boolean );
(* a < b ? *)
var   code : integer (* 0..3 *);
begin   arithpars ( 2, code );
        if not syserror then
                if code <> 3 then syserror:= true       (* 2 ints needed *)
(*11*)          else success :=  sparv [ 1 ]  <  sparv [ 2 ]
end;

procedure nameinf ( var success : boolean );
(* implement '@<' :  sysrout parameters are argless functors,
   check their lexicographic ordering.                          *)
begin   if not isargless ( spar [ 1 ] ) or not ( isargless ( spar [ 2 ] ) )
                                                        then syserror:= true
        else success:= compstrings( at [ mt [ spar [ 1 ] ] ]. name,
                                    at [ mt [ spar [ 2 ] ] ]. name,  false )
end;

procedure ordchr ( var success : boolean );
(* chr ( a mod nch ) = x  or  x = ord ( b )  or  a mod nch = ord ( b ) ?
   nch is the number of characters, i.e. maxchar + 1 ( 128 for ASCII )   *)
var   ch : char;   isch : boolean;
begin   isch:= ischar ( spar [ 2 ] , ch );      (* ch defined if isch *)
        if isint ( spar [ 1 ] ) then
                if isvar ( spar [ 2 ] ) then
                        bindpar( 2, charprot( chr( sparv[1] mod (maxchar+1))))
                else
                if isch then success:= sparv[1] mod (maxchar+1) = ord ( ch )
                else syserror:= true
        else
        if isch and isvar ( spar [1] ) then arithans ( 1, ord ( ch ) )
        else syserror:= true
end (*ordchr*);

procedure ch1class ( class : classofchar;   var success : boolean );
(* if the sysrout parameter is not a character, fail.
   otherwise check if it is of this class.                 *)
var   c : char;
begin   if ischar ( spar [ 1 ], c ) then success:= cc [ c ] = class
        else success:= false
end;

procedure ch2class ( class : classofchar;   var success : boolean );
(* like chclass1, but c < class is also ok *)
var   c : char;
begin   if ischar ( spar [ 1 ], c ) then success:= cc [ c ] <= class
        else success:= false
end;





(*---------  s t r u c t u r e   a c c e s s  ---------*)

function atomspar ( name, arity : nsysparam )  : atx;
(* extract an atom description from a sysrout call specifying name & arity.
   -----> used also outside structure access <-----                        *)
begin   if not isargless( spar [name]) or not isint( spar [arity]) then begin
                syserror:= true;   atomspar:= atlow     (* a dummy value *)
        end
        else atomspar:= samename ( mt [spar [name]] , sparv [arity] )
end;

procedure eqvar ( var success : boolean );
(* are both sysrout parameters the same non-dummy variable ? *)
begin   if not isndvar( spar[1]) or not isndvar( spar[2]) then success:= false
        else success :=  spar [ 1 ] = spar [ 2 ]
end;

procedure functor ( var success : boolean );
(* implement  functor( term, func, narity )  *)
var   atom : atx;
begin   if isvar ( spar [ 1 ] ) then      (* construct most general term *)
                if isint ( spar [ 2 ] ) then
                        if not isint ( spar [ 3 ] ) then syserror:= true
                        else
                        if sparv [ 3 ] <> 0 then syserror:= true
                        else bindpar ( 1, spar [ 2 ] )
                else begin
                        atom:= atomspar ( 2, 3 );
                        if not syserror then bindpar ( 1, gterm ( atom ) )
                end
        else    (* match term's functor with arity and func  *)
        if isint ( spar [ 1 ] ) then
                if unify ( spar [ 2 ], ancenv, spar [ 1 ], vtop ) then
                        success:= unify( spar[3], ancenv, heapint( 0 ), vtop )
                else success:= false
        else                                    (* try narity, then func *)
        if unify( spar[3], ancenv, heapint( getarity( spar[1] ) ), vtop ) then
                success:= unify( spar [ 2 ], ancenv,
                                 gterm( samename( mt [ spar[1]] ,0 ) ), vtop )
        else success:= false
end (*functor*);

procedure arg ( var success : boolean );
(* implement  arg ( number, term, argument )  *)
begin   if not isint( spar[1]) or not is_normal( spar [2]) then syserror:= true
        else
        if (sparv[1]<1) or (sparv[1] > getarity(spar[2])) then success:= false
        else success:= unify ( spar [ 3 ], ancenv,
                                argument (spar[2], ancenv, sparv[1]), ancenv )
end (*arg*);

function intexplode ( n : integer )  : mtx;
(* produce a list of n's digits ( in character form ) *)
var   list : mtx;
begin   list:= nilprotx;
        repeat  list:= heapdot( charprot( chr( n mod 10 + ord('0'))), list );
                n:= n div 10
        until n = 0;
        intexplode:= list
end;

function nameexplode ( firstc : ctx )  : mtx;
(* produce a list of this printname's characters *)
var   list : mtx;   lastc : ctx;
begin   list:= nilprotx;
        for lastc:= charlast ( firstc ) downto firstc do
                list:= heapdot ( charprot ( ct [ lastc ] ), list );
        nameexplode:= list
end;

function trydigseq ( list : mtx; env : stackx; var num : integer )  : boolean;
(* if list is a digit sequence ( in character form ), convert it to num,
 otherwise yield false. an empty list is  n o t  a digit sequence !     *)
var   is : ( no, dontknow, yes );   dig : char;   d : integer;
      err, turn1 : boolean;
begin   is:= dontknow;   err:= false;   num:= 0;   turn1:= true;
        repeat  if not is_normal ( list ) then is:= no
                else
                if mt [ list ] = std [ atmnil ] then
                        if turn1 then is:= no
                        else is:= yes
                else
                if mt [ list ] <> std [ atmdot ] then is:= no
                else begin     (* ischar defines dig for all branches ! *)
                        if not ischar( argument( list, env, car ), dig ) then
                                                                       is:= no
                        else
                        if ( dig < '0' ) or ( dig > '9' ) then is:= no
                        else
                        if num > maxint div 10 then err:= true
                        else begin
                                num:= num * 10;   d:= ord( dig ) - ord( '0' );
                                if num <> 0 then
                                        if ( maxint div num = 1 ) and
                                           ( d > maxint mod num ) then
                                                                   err:= true;
                                if not err then num:= num + d
                        end;
                        list:= argument ( list, env, cdr );      (*tail*)
                        turn1:= false
                end
        until ( is <> dontknow ) or err;
        trydigseq :=  is = yes
end;

function trycharseq ( list : mtx;  env : stackx;  var atom : atx )  : boolean;
(* if list is a character sequence, construct a name and return
   its atom ( null arity ).  otherwise yield false.
   an empty list  i s  a character sequence !                     *)
var   is : ( no, dontknow, yes );   c : char;
begin   is:= dontknow;
        repeat  if not is_normal ( list ) then is:= no
                else
                if mt [ list ] = std [ atmnil ] then is:= yes
                else
                if mt [ list ] <> std [ atmdot ] then is:= no
                else begin
                        if ischar ( argument ( list, env, car ), c ) then
                                                               buildname ( c )
                        else is:= no;
                        list:= argument ( list, env, cdr )      (*tail*)
                end
        until is <> dontknow;
        if is = no then ctop:= builtname        (* restore dictionary state *)
        else atom:= findatom ( wrapname, 0 );
        trycharseq :=  is = yes
end;

procedure pname ( var success : boolean );
(* implement  pname( functor, charlist )  *)
var   list : mtx;   atom : atx;
begin   if not isvar ( spar [ 1 ] ) then                (* explode *)
                if not isargless ( spar [ 1 ] ) then syserror:= true
                else begin
                        list:= nameexplode ( at [ mt [ spar [ 1 ] ] ] . name );
                        success:= unify ( spar [ 2 ], ancenv, list, vtop )
                end
        else            (* construct after checking for non-empty list *)
        if not is_normal ( spar [ 2 ] ) then syserror:= true
        else
        if mt [ spar [ 2 ] ]  <>  std [atmdot] then syserror:= true
        else
        if trycharseq( spar[2], ancenv, atom ) then bindpar( 1, gterm( atom ))
        else syserror:= true
end (*pname*);

procedure pnamei ( var success : boolean );
(* implement  pnamei( integer, digitlist )  *)
var   num : integer;
begin   if not isvar ( spar [ 1 ] ) then               (* explode *)
                if not isint ( spar [ 1 ] ) then syserror:= true
                else success:= unify ( spar [ 2 ], ancenv,
                                       intexplode ( sparv [ 1 ] ), vtop )
        else
        if trydigseq ( spar [ 2 ], ancenv, num ) then
                bindpar ( 1, heapint ( num ) )
        else syserror:= true
end (*pnamei*);






(*----------  c l a u s e / a s s e r t / r e t r a c t  ----------*)

         (* assert uses free stack storage for temporary data *)



function varconvert ( v : stackx )  : varnumb;
(* v is a variable instance. return its number in temporary
   variable dictionary, inserting it there if missing.          *)
var   p : stackx;   found : boolean;
begin   p:= vardictbot;   found:= false;
        while ( p <> vtop ) and not found do
                if mt [ p ] =  -v then found:= true
                else p:= p + 1;
        if not found then begin                      (* p = vtop *)
                pushvars ( 1 );   mt [ p ]:=  - v
        end;
        varconvert:= p - vardictbot
end;

function storeterm ( pt : mtx;  env : stackx )  : protx;
(* used only by storeclause. pt is a non-variable term instance or
   prot in variable environment env. store it as a prototype, using
   temporary variable dictionary to renumber variables.                 *)
var   p : protx;   k : argnumb;   arg : mtx;
begin   (* check (10, not isndvar ( pt ) ); *)
        if ( pt >= protlow ) and ( pt < gtop ) then storeterm:= pt (*ground*)
        else                            (* heap integer still possible *)
        if isint ( pt ) then storeterm:= newintprot ( intval ( pt ) )
        else begin
                p:= initprot ( mt [ pt ] );
                for k:= 1 to getarity ( pt ) do begin
                        arg:= argument ( pt, env, k );
                        if isndvar ( arg ) then
                                newpvararg( p,k,varconvert( varadr( arg,env)))
                        else newparg ( p, k, storeterm ( arg, env ) )
                end (*for*);
                storeterm:= wrapprot ( p )
        end
end;

function storeclause ( headpt, bodypt : mtx;  env : stackx )  : procptr;
(* headpt and bodypt are correct terms ( or prots in env ) representing
   a clause head and body ( cf.assert ).  store this clause.              *)
var   p : procptr;
begin   vardictbot:= vtop;      (* variable dictionary used by storeterm *)
        new ( p );
        with p ^  do begin
                next:= nil;   systemroutine:= false;
                head:= storeterm ( headpt, env );
                body:= storeterm ( bodypt, env );
                nvars:= vtop - vardictbot       (*no. of vars in dictionary*)
        end;
        vtop:= vardictbot;   storeclause:= p
end;

function procok ( headatom : atx )  : boolean;
(* if headatom is associated with a system routine or protected procedure
   (but single-letter argless predicates can't be protected!),
   yield false & a warning                                                    *)
var   ok, keep : boolean;
begin   with at [ headatom ] do begin
                if proc = nil then ok:= true
                else
                if proc ^. systemroutine then ok:= false
                else with proc^ do
                        ok:= ( (head >= gprotected) and (head <= ngprotected) )
                             or ((head >= minchprotx) and (head <= maxchprotx));
                if not ok then begin
                        keep:= telling;   telling:= false;   putline;
                        write ( '-----redefinition of system routine ',
                                    'or protected procedure ignored :'     );
                        putline;   putname ( headatom );   putch( '/' );
                        putint ( arity );   putline;   telling:= keep
                end
        end (*with*);
        procok:= ok
end;

procedure linkclause ( cl : procptr;  n : integer );
(* insert cl after the nth clause in its procedure ( as first if n <= 0 ,
   as last if n too large). ** note the effect of redefining ** *)
var   headatom : atx;   p : procptr;
begin   headatom:= mt [ cl ^. head ];
        if procok ( headatom ) then begin
                if redefining and ( headatom <> lastasserted ) then begin
                        lastasserted:= headatom;   at [ headatom ]. proc:= nil
                end;
                p:= at [ headatom ]. proc;
                if ( n <= 0 ) or ( p = nil ) then       (*insert as first*)
                        with at [ headatom ] do begin
                                cl^.next:= proc;   proc:= cl
                        end
                else begin      (* find clause n  (or last) to link to *)
                        while ( n <> 1 ) and ( p^.next <> nil ) do begin
                                n:= n - 1;   p:= p^.next
                        end;
                        cl^.next:= p^.next;   p^.next:= cl
                end
        end
end (*linkclause*);

procedure assert;
(* implement  assert( head, body, number )  :
   construct a clause and invoke linkclause.    *)
begin   if not is_normal(spar[1]) or not isint(spar[3]) then syserror:= true
        else
        if not iscallsequence ( spar [ 2 ], ancenv ) then syserror:= true
        else  linkclause ( storeclause( spar[1], spar[2], ancenv ), sparv[3] )
end;

procedure clause ( var success : boolean );
(* implement  clause( functor, arity, n, head, body) . the first 3 are
   given. find the n'th clause in the procedure associated with functor &
   arity (failing if none) and try to unify it with head and body.        *)
var   atom : atx;   p : procptr;   k : integer;
begin   if not isint(spar[3]) then syserror:= true
        else atom:= atomspar ( 1, 2 );     (* checks parameters, remember ? *)
        if not syserror then begin                      (* call ok *)
                p:= at [ atom ]. proc;   k:= 1;
                while ( p<> nil ) and ( k < sparv [ 3 ] ) do begin
                        p:= p^.next;   k:= k + 1
                end;
                if ( p = nil ) or ( sparv [ 3 ] < 1 ) then success:= false
                else
                if p^.systemroutine then success:= false
                else begin      (* create fresh clause instance and unify *)
                        (*check (11, topenv = vtop );*)   pushvars ( p^.nvars );
                        success:= unify( spar[ 4 ], ancenv, p^.head, topenv );
                        if success then         (*heads unified, bodies now *)
                                success:= unify ( spar [ 5 ], ancenv,
                                                        p^.body, topenv )
                end
        end (*call ok*)
end (*clause*);

procedure genproc ( var success : boolean );
(* implement  $proc( head ) , an auxiliary of  proc( head )  ( which is
   written in prolog ) . try to unify with the most general term instance
   of the atom shown by procindx (cf. proclimit).                         *)
begin   success:= unify ( spar [ 1 ], ancenv, gterm ( procindx ), vtop )
end;

procedure proclimit ( var success : boolean );
(* implement  $proclimit , a shameful auxiliary of  proc( head ) .
   set procindx (which is initialised by  $procinit , cf. sysroutcall)
   to the next atom associated with a non-system procedure and  f a i l .
   (succeed if no such atoms left).                                         *)
var   found : boolean;
begin   found:= false;
        while ( procindx < athigh ) and not found do begin
                procindx:= procindx + 1;
                with at [ procindx ] do
                        if name <> emptyatom then
                                if proc <> nil then
                                        found:= not proc^.systemroutine
        end;
        success:= not found
end;

procedure retract ( var success : boolean );
(* implement  retract( name, arity, number ) :  retract the number'th
   clause of those whose head predicate has this name and arity,
   fail if none such.                                                   *)
var   atom : atx;   p : procptr;   n : integer;
begin   atom:= atomspar ( 1, 2 );
        if not syserror then syserror:= not procok ( atom )  or
                                                not isint ( spar [ 3 ] );
        if not syserror then begin                              (* call ok *)
                n:= sparv [ 3 ];
                if n < 1 then success:= false
                else
                if n = 1 then           (* remove first clause *)
                        with at [ atom ] do
                                if proc = nil then success:= false
                                else proc:= proc^.next
                else begin              (* find the preceding clause *)
                        p:= at [ atom ]. proc;
                        while ( n <> 2 ) and ( p <> nil ) do begin
                                n:= n - 1;   p:= p^. next
                        end;
                        if p = nil then success:= false        (*no (n-1)'th*)
                        else with p^  do
                                if next <> nil then next:= next ^. next
                end (*not 1st*)
        end (*call ok*)
end (*retract*);

procedure abolish;
(* implement  abolish( name, arity )  *)
var   atom : atx;
begin   atom:= atomspar ( 1, 2 );
        if not syserror then if procok ( atom ) then at [ atom ]. proc:= nil
end;

procedure redefine;
(* tag used as nil ( tag can't be redefined ) *)
begin   redefining:= not redefining;
        if not redefining then lastasserted:= std [atmtag]
end;

procedure predefined ( var success : boolean );
(* implement  predefined( name, arity ) :  is this a system routine ? *)
var   atom : atx;
begin   atom:= atomspar ( 1, 2 );
        if not syserror then
                with at [ atom ] do
                         if proc = nil then success:= false
                         else success:= proc ^. systemroutine
end;


procedure protect;
(* set protected boundaries, so that clauses which are already defined
   will be protected by procok                                          *)
begin   gprotected := gtop;   ngprotected := ngbot
end;





(*---------  h i g h  i n p u t / o u t p u t  ---------*)



procedure rdchar;
(* read the next character into cch ( linend returned as chr(eol) ).
   non-user file end redirects input stream to user, user file end is fatal.*)
begin   while endoffile do
                if sysloading or seeing then begin
                        closefile ( true );   writeln;
                        writeln ( ' -----end of file, terminal input now' );
                        putline
                end
                else errror ( usereof );
        if endofline then begin
                getline;   cch:= chr ( eol )
        end
        else getch ( cch )
end (*rdchar*);

procedure lastch ( var success : boolean );
(* try to unify sysrout parameter with current character.
   ( special-case unification coded here for efficiency ) *)
var   c : char;
begin   if isvar ( spar [ 1 ] ) then bindpar ( 1, charprot ( cch ) )
        else
        if ischar ( spar [ 1 ] , c ) then success :=  cch = c
        else success:= false
end;

procedure skipbl;
(* ensure that cch is not filled with a white space character *)
begin   while cch <= ' ' do rdchar              (*** ASCII version ***)
end;

procedure wch;
(* output the sysrout parameter, which should be a character term *)
var   c : char;
begin   if ischar ( spar [ 1 ], c ) then
                if c = chr ( eol ) then putline
                else putch ( c )
        else syserror:= true
end;

procedure newfile ( see : boolean );
(* implement see or tell, according to parameter *)
begin   if not isargless ( spar [ 1 ] ) then syserror:= true
        else
        if mt [ spar [ 1 ] ]  =  std [atmuser] then
                closefile ( see )
        else begin
                openfile ( at [ mt [ spar [ 1 ] ] ] . name ,  see );
                if see then seeobj:= spar [ 1 ]
                else tellobj:= spar [ 1 ]
        end
end (*newfile*);

procedure whatfile ( see : boolean;  var success : boolean );
(* implement seeing or telling, according to parameter *)
var   fileterm : mtx;
begin   if see then fileterm:= seeobj
        else fileterm:= tellobj;
        success:= unify ( spar [ 1 ] , ancenv, fileterm, vtop )
end;




(*---------  s y s t e m  r o u t i n e s  ---------*)



procedure haltaction ( var stop : boolean );
(* display the sysparam and halt the interpreter *)
begin   putline;   display ( spar [ 1 ] , ancenv );   putline;
        stop:= true;   terminate:= true
end;

procedure sysroutcall (* ( id : sysroutid;  var success, stop : boolean ) *);
(* perform a system routine call *)
var   k : nsysparam;
begin   syserror:= false;   success:= true;     (* might change yet *)
        for k:= 1 to getarity ( ccall ) do begin
                spar [ k ] :=  argument ( ccall, ancenv, k );
                if isint( spar [ k ] ) then sparv [ k ]:= intval( spar [ k ] )
        end;
        case id of
        idfail          : success:= false;      (* keep this as first  *)
        idtag ,
        idcall          : ;             (* never called ! (cf. control) *)
        idslash         : slash;
        idtagcut        : tagcut ( success );
        idtagfail       : tagfail ( success );
        idtagexit       : tagexit ( success );
        idancestor      : ancestor ( success );
        idhalt          : haltaction ( stop );
        idstatus        : status;
        iddisplay       : display ( spar [ 1 ], ancenv );
        idrch           : rdchar;
        idlastch        : lastch ( success );
        idskipbl        : skipbl;
        idwch           : wch;
        idecho          : echo:= true;
        idnoecho        : echo:= false;
        idsee           : newfile ( true );
        idseeing        : whatfile ( true, success );
        idseen          : closefile ( true );
        idtell          : newfile ( false );
        idtelling       : whatfile ( false, success );
        idtold          : closefile ( false );
        idordchr        : ordchr ( success );
        idsum           : sum ( success );
        idprod          : prod ( success );
        idinf           : inf ( success );
        idnameinf       : nameinf ( success );
        idbigletter     : ch1class ( cbigletter, success );
        idsmalletter    : ch1class ( csmalletter, success );
        idletter        : ch2class ( csmalletter, success );
        iddigit         : ch1class ( cdigit, success );
        idalphanum      : ch2class ( cunderscore, success );
        idbracket       : ch1class ( cbracket, success );
        idsolochar      : ch1class ( csolochar, success );
        idsymch         : ch1class ( csymch, success );
        ideqvar         : eqvar ( success );
        idvar           : success:= isvar ( spar [ 1 ] );
        idatom          : success:= isargless( spar [1] );
        idinteger       : success:= isint ( spar [ 1 ] );
        idnonvarint     : success:= is_normal ( spar[ 1 ] );
	idfunctor       : functor ( success );
	idarg           : arg ( success );
	idpname         : pname ( success );
	idpnamei        : pnamei ( success );
	idproc          : genproc ( success );
	idproclimit     : proclimit ( success );
	idprocinit      : procindx:= atlow - 1;
	idclause        : clause ( success );
	idretract       : retract ( success );
	idabolish       : abolish;
	idassert        : assert;
	idredefine      : redefine;
	idpredefined    : predefined ( success );
	idprotect       : protect;
	idnonexistent   : shownonexistent:= true;
	idnononexistent : shownonexistent:= false;
	iddebug         : debug:= true;
	idnodebug       : debug:= false;
	idtrue          : (*no action*)         (* keep this as last  *)
	end (*case*)
end (*sysroutcall*);





(*---------  r e a d e r  ---------*)

	(* frame variable area is used as an auxiliary argument stack *)



procedure rd;
(* the reader's input routine :  like rdchar, but skips linends *)
begin   repeat  rdchar
	until cch <> chr ( eol )
end;

procedure skipcombl;
(* skip white space and comments. a comment is a sequence between a "%"
   and a linend ( outside clauses or following dots and head colons )   *)
begin   skipbl;
	while cch = '%' do begin
		repeat rdchar
		until cch = chr ( eol );
		rd;  skipbl
	end
end;

procedure synterr;
(* produce a syntactic error message and raise recoverable error
   ( the reader does not normally process hand-produced code ) *)
var   keep : boolean;
begin   keep:= echo;   echo:= true;
	while cch <> chr ( eol ) do rdchar;             (* skip this line *)
	echo:= keep;   errror ( whaterr )
end;

function rddigits  : integer;
(* read a non-empty sequence of digits, return the integer value *)
(* overflow checking as in trydigseq, cf. comment to call in rdterm *)
var   v, d : integer;
begin   skipbl;   v:= 0;
	if cc [ cch ] <> cdigit then synterr;
	repeat  d:= ord ( cch ) - ord ( '0' );  (* value of digit *)
		if v > maxint div 10 then synterr;
		v:= v * 10;
		if v <> 0 then
			if ( maxint div v = 1 ) and ( d > maxint mod v ) then
								      synterr;
		v:= v + d;
		rd
	until cc [ cch ] <> cdigit;
	rddigits:= v
end;

function rdfunctor  : atx;
(* read a functor and return name atom pointer *)
const quote = '''';
var   stop : boolean;
begin   skipbl;
	if cc [ cch ] <= csmalletter then               (* i.e. a letter *)
		repeat buildname ( cch );  rd
		until cc [ cch ] > cdigit       (*i.e. not a letter or digit*)
	else
	if cch = '[' then begin         (* [], i.e. a "nice" nil *)
		rd;   if cch <> ']' then synterr;   rd;
		buildname ( '[' );   buildname ( ']' )
	end
	else
	if cch = '!' then begin                 (* a "nice" cut *)
		buildname ( cch );   rd
	end
	else begin
		if cch <> quote then synterr;
		stop:= false;
		repeat  rd;
			if cch <> quote then buildname ( cch )
			else begin      (* either last "'" or "''",ie. quote*)
				rd;
				if cch = quote then buildname ( quote )
				else stop:= true
			end
		until stop
	end (*else*);
	rdfunctor:= wrapname
end;

function rdterm  : integer;   forward;  (*--------------------------*)

procedure mkarg ( p : protx;  n, arg : integer );
(* insert arg as n'th in prot p. arg <= 0 encodes variable offset -arg. *)
begin   if arg <= 0 then newpvararg ( p, n, - arg )
	else newparg ( p, n, arg )
end;

function rdnonvarint  : protx;
(* read a non-variable & non-integer term, return its prototype *)
var   name : atx;   toplev : stackx;   prot : protx;   k : integer;
begin   toplev:= vtop;                          (* auxiliary stack level *)
	name:= rdfunctor;   skipbl;
	if cch = '(' then begin         (* read and stack the arguments *)
		repeat  rd;   pushvars ( 1 );
			mt [ vtop - 1 ] :=  rdterm;   skipbl
		until cch <> ',';
		if cch <> ')' then synterr
		else rd
	end;
	(* number of arguments known now :  construct the prototype *)
	prot:= initprot ( findatom ( name, vtop - toplev ) );
	for k:= 1 to vtop - toplev do mkarg ( prot, k, mt [ toplev-1+k ] );
	vtop:= toplev;   rdnonvarint:= wrapprot ( prot )
end;

function rdterm (*  : integer *);
(* read a term and return a prot for a non-var or a negated offset for a var.
   sequences processed recursively to allow proper ground prot treatment.   *)
var   sign : -1..1;   varoff : varnumb;   prot : integer;   dot : protx;
begin   skipbl;
	if cch = '(' then begin         (* eg.  a . (b . c) . d *)
		rd;   prot:= rdterm;   skipbl;
		if cch <> ')' then synterr;   rd
	end
	else
	if cch = '_' then begin                 (* a dummy variable *)
		rd;   prot:= dumvarx            (* treated as non-var here *)
	end
	else
	if cch = ':' then begin                 (* a variable *)
		rd;   varoff:= rddigits;   prot:= - varoff;
		if varoff + 1 > nclvars then nclvars:= varoff + 1
	end
	else
	if ( cch = '+' ) or ( cch = '-' ) or ( cc [cch] = cdigit ) then begin
		if cch = '-' then sign:= -1 else sign:= 1;
		if cc [ cch ] <> cdigit then rd;
		(* number itself processed as positive :  this
		   causes loss of smallest integer in two's complement *)
		prot:= newintprot ( sign * rddigits )
	end
	else prot:= rdnonvarint;
	skipbl;
	if cch <> '.' then rdterm:= prot
	else begin                      (* a sequence, as it turns out *)
		dot:= initprot ( std [atmdot] );   mkarg ( dot, car, prot );
		rd;   skipcombl;
		mkarg ( dot, cdr, rdterm );   rdterm:= wrapprot ( dot )
	end
end (*rdterm*);

procedure rdclause ( var isgoal : boolean;  var nvars : varnumb;
						var head, body : protx );
(* read a clause.
  ( a goal has an empty head and is terminated by a "#" )       *)
begin   nclvars:= 0;            (* number of variables in this clause *)
	vtop:= mtlow;   csbot:= stackhigh;      (*initialise argument stack*)
	skipcombl;
	if cch = ':' then begin
		isgoal:= true;   head:= nilprotx
	end
	else begin
		isgoal:= false;   head:= rdnonvarint;   skipbl;
		if cch <> ':' then synterr
	end;
	rd;  skipcombl;   body:= rdterm;   nvars:= nclvars;
	if isgoal then begin (*"#" terminates a goal,so resolution can
				begin before more input is requested.   *)
		skipbl;   if cch <> '#' then synterr;
		cch:= ' '  (*std 1st lastch, 1st rch gets char following "#"*)

	end
end (*rdclause*);

procedure readtogoal ( var goalsequence : protx;  var nvarsingoal : varnumb );
(* read clauses until nearest goal statement :  return its attributes *)
var   isgoal : boolean;   locproc : proc;   p : procptr;
begin   locproc.next:= nil;   locproc.systemroutine:= false;
	repeat  with locproc do begin   (*read clause, iscallsequence checks*)
			rdclause ( isgoal, nvars, head, body );
			vtop:= mtlow;
			pushvars ( nvars );   (*dummy env for iscallsequence*)
			if not iscallsequence ( body, mtlow ) then synterr
		end;
		if isgoal then begin
			goalsequence:= locproc.body;
			nvarsingoal:= locproc.nvars
		end
		else begin      (* note that mthigh > no. of clauses *)
			new ( p );   p^:=  locproc;   linkclause ( p, mthigh )
		end
	until isgoal
end (*readtogoal*);

function rdatom  : atx;
(* read an atom description ( during sysloading ). format : functor/arity  *)
var   name : atx;
begin   skipcombl;   name:= rdfunctor;
	if cch <> '/' then synterr;
	rd;   rdatom:= findatom ( name, rddigits )
end;





(*---------  i n i t i a l i s a t i o n  ---------*)



procedure initvars;
(* initialise global variables (cf. startresolution & loadsyskernel) *)
var   a : atx;   c: char;
begin 
	colinesize:= 0;   solinesize:= 0;
	seeing:= false;   telling:= false;
	ctop:= ctlow;   builtname:= ctlow;
	freeatoms:= athigh - atlow + 1;
	for a:= atlow to athigh do at [ a ]. name:= emptyatom;
(* ???  ct [ emptyatom ] :=  chr( eos );  ??? *)

        if stdintprotx+(maxstdint-minstdint) >= mthigh then errror ( sysinit );
	mt [ dumvarx ] :=  varlim;
	gtop:= stdintprotx;     (* loadsyskernel picks gtop up from here *)
	ngbot:= mthigh;
	gprotected:= gtop;   ngprotected:= ngbot;
	tminfree:= tthigh;   bminfree:= bthigh;
	fminfree:= fthigh;   stackminfree:= stackhigh - mtlow;
	terminate:= false;
	redefining:= false;
	shownonexistent:= false;      (* don't warn on calls to nonexistent *)
	cch:= ' ';   echo:= false;   debug:= false;
	(* character classification, ASCII version.
	   symch is a non-alphameric freely used to form symbols.  *)
	for c:= chr ( 0 ) to chr ( maxchar ) do cc [ c ] := cother;
	for c:= 'a' to 'z' do cc [ c ] := csmalletter;
	for c:= 'A' to 'Z' do cc [ c ] := cbigletter;
	for c:= '0' to '9' do cc [ c ] := cdigit;
	cc [ '(' ] := cbracket;   cc [ ')' ] := cbracket;
	cc [ '[' ] := cbracket;   cc [ ']' ] := cbracket;
	cc [ '{' ] := cbracket;   cc [ '}' ] := cbracket;
	cc [ ',' ] := csolochar;
	cc [ ';' ] := csolochar;   cc [ '!' ] := csolochar;
	cc [ '_' ] := cunderscore;
	cc [ '+' ] := csymch;   cc [ '-' ] := csymch;
	cc [ '*' ] := csymch;   cc [ '/' ] := csymch;
	cc [ '=' ] := csymch;   cc [ '@' ] := csymch;
	cc [ '#' ] := csymch;   cc [ '$' ] := csymch;
	cc [ '&' ] := csymch;   cc [ '.' ] := csymch;
	cc [ ':' ] := csymch;   cc [ '?' ] := csymch;
	cc [ '<' ] := csymch;   cc [ '>' ] := csymch;
	cc [ '^' ] := csymch;   cc [ '\' ] := csymch;
end (*initvars*);

procedure loadsyskernel;
(* create standard objects.
   kernel library will be processed in main program loop.    *)
var   k : integer;   a : stdatomid;   s : sysroutid;
      p : procptr;   prot : protx;
begin   sysloading:= true;   whaterr:= loadfile;
        assign ( mc, 'syskernel' );
        reset ( mc );
        for a:= atmsemcol to atmuser do std [ a ] :=  rdatom;
	lastasserted:= std [atmtag];  (*cf. "redefine"*)
	for s:= idfail to idtrue do begin
		new ( p );
		with p^ do begin
			next:= nil;   nvars:= 0;
			systemroutine:= true;   sysid:= s
		end;
		at [ rdatom ]. proc:= p         (* NOTE the function call *)
	end (*for*);
	whaterr:= syntax;
	(* create nil, character & standard integer prototypes *)
	mt [ nilprotx ] :=  std [atmnil] ;
	for k:= 0 to maxchar do begin
		buildname ( chr ( k ) );
		mt [ minchprotx + k ] :=  findatom ( wrapname, 0 )
	end;
	for k:= minstdint to maxstdint do mkint ( k );
	(* create error call sequence & user file name prot *)
	prot:= initprot ( std [atmerror] );   newpvararg ( prot , 1, 0 );
	prot:= wrapprot ( prot );
	errorcallseq:= initprot ( std [atmdot] );
	newparg(errorcallseq,1,prot );   newparg(errorcallseq, 2, nilprotx);
	errorcallseq:= wrapprot ( errorcallseq );
	userprot:= wrapprot ( initprot ( std [atmuser] ) );
	(* mc is a special stream. set default streams for when it ends *)
	seeobj:= userprot;   tellobj:= userprot
end (*loadsyskernel*);


begin (*********** toy prolog ************)
	initvars;
	loadsyskernel;
2:      repeat  readtogoal ( goalstmnt, ngoalvars );
		resolve ( goalstmnt, ngoalvars )
	until terminate;
1:      closefile ( true );   closefile ( false )
end.
