#!/bin/sh

#############################################################

# run pascal rejection tests

#############################################################

ldiff() {
    diff --text $1 $2 | less
}

# use this difference program
if [ -n "$DISPLAY" ] && which kdiff3 > /dev/null 2>&1; then
    DIFF=kdiff3
else
    DIFF=ldiff
fi

echo -n "iso 7185 acceptance and  tests"; pwd
echo

if [ -z $P5CDIR ] ; then
   echo "\$P5CDIR is not set, bailing out"
   exit 1;
fi

dcheck() {
  if [ -f $PASFILE.out ]
  then
    if diff -qb --ignore-matching-lines="INFO" $PASFILE.out $PASFILE.cmp ; then
       printf "\r$PASFILE is OK "
    else
       $DIFF $PASFILE.out $PASFILE.cmp
    fi
  else
    echo "failed: $PASFILE did not produce an output"
    printf 'Press RETURN to continue ... '
    read REPLY
  fi
}

rm -f *.out *.lst *.err *.1.pas *.p *.c *.exe


echo
echo "acceptance tests"
rm -f iso7185pat iso7185pat.exe
$P5CDIR/pc iso7185pat
./iso7185pat < iso7185pat.in | awk -f patCheck.awk | tail -4 | tee iso7185pat.result

printf 'Press RETURN to continue ... '
read REPLY

echo
echo
for i in iso7185prt*[!.]?.pas; do
  PASFILE=`basename $i .pas`
  rm -f $PASFILE $PASFILE.out $PASFILE.lst
  echo -n "$PASFILE "
  $P5CDIR/r $PASFILE 2> /dev/null > $PASFILE.out
  if [ -f $PASFILE.exp ] ; then

      if diff -qb -I 'P5. Pascal compiler v' $PASFILE.lst $PASFILE.exp ; then
          echo "compile time check OK"
      else
          $DIFF $PASFILE.lst $PASFILE.exp
      fi
  else
      #### check it compiled OK ###
      if grep -l 'Errors in program: 0' $PASFILE.lst > /dev/null 2>&1 ; then

          if diff -qb -I '++IGNORE++' $PASFILE.out $PASFILE.cmp ; then
              echo "runtime check OK"
          else
              $DIFF $PASFILE.out $PASFILE.cmp
          fi
      else
          echo "unexpected compilation error in $PASFILE"
          $DIFF $PASFILE.lst $PASFILE.exp
      fi
  fi
done

echo
echo "`ls iso7185prt*[!.]?.pas | wc -w` tests"
echo '---------------'
awk ' BEGIN { n=0; } \
     /error not detected/ { a[n++] = FILENAME ": " $0 }; \
     END {print n " tests did not detect an error:"; \
     for(i=0;i<n;i++) print a[i]; } ' iso7185prt*[!.]?.out

echo '---------------'
awk ' BEGIN { n=0; } \
     /deviation/ { a[n++] = FILENAME ": " $0 }; \
     END {print "deviations found in " n " tests:"; \
     for(i=0;i<n;i++) print a[i]; } ' iso7185prt*[!.]?.out

echo '---------------'
awk ' BEGIN { n=0; } \
     /not an error/ { a[n++] = FILENAME ": " $0 }; \
     END {print "no p5c error found in " n " tests:"; \
     for(i=0;i<n;i++) print a[i]; } ' iso7185prt*[!.]?.out

echo '---------------'
awk ' BEGIN { n=0; } \
     /implementation defined:/ { a[n++] = FILENAME ": " $0 }; \
     END {print "implementation defined found in " n " tests:"; \
     for(i=0;i<n;i++) print a[i]; } ' iso7185prt*[!.]?.out



echo '---------------'
for i in *.lst ; do
    if ! grep -q "Errors in program:" $i ; then
        echo "$i did not compile";
    fi;
done

for i in [!x]*.err ; do
    if grep -q ": error:" $i ; then
        echo "bad c code in $i";
    fi;
done

grep 'Compiler internal error' *.lst
grep -l 'No such file or directory' *.out

# for debug version of pcom.pas
# show where memory leaks
echo "found `grep 'new memory at line' *.lst | wc -l` memory leaks"
cat *.lst | grep "new memory at line" | sort | uniq

printf 'Press RETURN to continue ... '
read REPLY

if which valgrind > /dev/null 2>&1; then

    MYDIR=$(pwd)
    #echo $MYDIR
    #echo ${MYDIR%/*}
    GENFILE=${MYDIR%/*}/gen

    export $(grep BUILD= $GENFILE)

    echo -n "waiting for $BUILD.t ... "
    while [ ! -x $P5CDIR/$BUILD.t ] ; do sleep 1; done
    echo "$BUILD valgrind tests ..."
    rm -f val-test.vlg
    for i in *.pas
    do
        printf "\r$i                        "
        valgrind -s --track-origins=yes --leak-check=full $P5CDIR/$BUILD.t $i 2>> val-test.vlg > /dev/null
    done
    echo

    awk 'BEGIN {nerr = 0;} \
         /Command:/ {cmd=$4;} \
         /HEAP SUMMARY:/   { \
           getline; if( !/ 0 bytes in 0 blocks/ ) { \
                        nerr++; \
                         print( cmd ": heap error") \
                    } \
           } \
         /ERROR SUMMARY:/ && !/0 errors/  { \
           nerr++; \
           print( cmd ": memory error" ) \
           } \
        END { \
           if( nerr == 0 ) \
              print( "valgrind found no errors"); \
           else if( nerr == 1) \
              print( "valgrind found one error"); \
           else \
              print( "valgrind found",  nerr,  "errors"); \
          } \
        ' val-test.vlg

    read -p 'Press RETURN to continue ... ' REPLY
    echo
fi
