{$d-,v-  no debug, so faster

#  // force c preprocessor

}
{%%%%%%%%% Copyright (c) 2020 Trevor Blight All rights reserved. %%%%%%%%%

   check how well a compiler handles bad code by mutating a test program

   mutate the test program, compile and report results

a model program is split into tokens, then mutate a program by
- remove a token completely
- change a token, replace it with some other tokem
- insert a new token

The mutated program is compiled, and the list file is checked for errors
This process is repeated for each token in the test program.

A token is:
- a sequence of alpha-numeric characters, or
- a single non-alpha-numeric character, or
- 'string of chars'
- whitespace is a token separator

NOTES:
- deleting "var" (as in a var parameter), "packed" & "not"
  doesn't (potentially) create errors.
  an instance of removing a "var" token to create an error would be if the var
  parameter contained a file.
  this type of error is checked in the standard validation/acceptance tests,
  and is beyond the scope of a mutation test.
  when one of these is encountered, the following token is removed as well.

TODO:
- -term does not create an error if the "-" is removed.  ditto plus
   compare with a - b, where removing the "-" does make an error

This program removes, replaces or inserts tokens
- other forms of mutation are not implemented.
  (Randomly swapping idenitifiers might be interesting.)

inputs, provided by the environment:
model        - the pascal source that is to be mutated
mutantSource - version mutated as described above
listfile     - result of compile step
//COMMAND    - c preprocessor command line that corresponds to these files
//SEED       - initial seed for add/replace with random tokens,

To reproduce a given test:
- create a version of mutate that contains the seed, action and
  default names for pas & list files, eg:

          ./pc -DACTION=add -DSEED=1339434591 mutate

- run this program with same file locations for pas & list files:

           ./mutate Test-File.pas /tmp/mut.pas /tmp/mut.lst


The model source goes into an array of lines, stripped of comments
each line is terminated by a null char

this program uses the c library and c preprocessor features of p5x pascal,
probably needs modifying for other pascal compilers

}
program mutate(input, output, model, mutantSource, listFile);

// system definitions
#include <sys.inc.pas>

const
   maxLines  = 10000;

{NB cmdStr needs a trailing char so system() can null terminiate it}
#if defined COMMAND
#define QUOTE() '
//'
   cmdStr = QUOTE()COMMAND QUOTE();
#else
   cmdStr = '$P5CDIR/p5x /tmp/mut.pas /tmp/mut.c > /tmp/mut.lst ';
#endif

type
   line    = packed array[1..200] of char;
   mutType = (remove, replace, add);   {remove/replace/add token}

   symbols = (ident, intconst, realconst, stringconst, notsy, mulop, addop, relop,
              lparent, rparent, lbrack, rbrack, comma, semicolon, period, arrow,
              colon, becomes, range, labelsy, constsy, typesy, varsy, funcsy, progsy,
              procsy, setsy, packedsy, arraysy, recordsy, filesy, beginsy, ifsy,
              casesy, repeatsy, whilesy, forsy, withsy, gotosy, endsy, elsesy, untilsy,
              ofsy, dosy, tosy, downtosy, thensy, nilsy, othersy);

var
   trap                          : boolean;  {for debug}
   model, mutantSource, listFile : text;
   tokens                        : array[1..maxLines] of line;
   nrLines                       : integer;  {nr lines }
   syStatus                      : integer;  {command status}
   lnr, pos                      : integer;  {mutant source file position}
   parenCount                    : integer;  {in parameter list?}

   now:real;

// include c functions
#include <clib.inc.pas>
#include <assert.inc.pas>


{
generate an initial seed from random contents of uninitialised memory
note that global memory is initialised to zero,
so we must use variables that are inside a function or procedure
}
procedure initAdd;
const max =  1024;
var
   a : array[1..max] of integer;
   i : integer;
   s : integer;
   k : integer;
begin
# if defined SEED
   s := SEED;
#else
   s := 0;
   for i := 1 to max do begin
      k := a[i];
      //writeln('s is ', s, ', a[', i, '] is ', a[i]);
      if k < -s then
         s := (k + maxint) + s
      else if k <= maxint - s then
         s := s + k
      else
         s := (s - maxint) + k;
      assert( s >= 0);
   end;
#endif

   writeln( 'initial rand seed is ', s:1 );
   srand(s);
end; { initAdd }


{read file model into tokens array
 strip comments
 convert to lower case
 each line is null terminated
}
procedure readSource;
var
   c,ch      : char;
   lout      : integer;  {line counter}
   pos       : integer;  {pos in line}
   inString  : boolean;
   lcase     : array[char] of char;

procedure skipComment;
var
   endComment :  boolean;
begin
   endComment := false;
   while not endComment and  not eof( model ) do begin
      if eoln( model ) then begin
         readln( model );
      end
      else begin
         read( model, ch );
         if ch = '}' then
            endComment := true
         else if (ch = '*') and (model^ = ')') then begin
            read( model, ch );
            endComment := true
         end; {if}
      end; {if}
   end; {while}
   ch := ' '
end; { skipComment }

begin {readSource}

   for c := chr(0) to chr(ORD_CHAR_MAX) do
      if c in ['A' .. 'Z'] then
         lcase[c] := chr( ord(c) - ord('A') + ord( 'a' ) )
      else
         lcase[c] := c;

   reset( model );
   if eof( model ) then begin
      writeln;
      writeln('usage:');
      writeln('mutate model mutantSource listfile');
      writeln;
      writeln('where:');
      writeln('      model is file that is to be tested');
      writeln('      mutantSource is mutated source file');
      writeln('      listfile is listing of compiled mutantSource');
      writeln('mutantSource and listfile must be compatible with cmdStr');
      writeln('see comments at head of source file for more details');
      writeln;
   end;
   pos := 1;
   lout := 1;
   inString := false;
   while not eof( model ) do begin
      if eoln( model ) then begin
         tokens[lout,pos] := chr(0);
         //writeln('line ', lout, ' ''', tokens[lout], '''');
         readln( model );
         lout := lout + 1;
         pos := 1;
      end
      else begin
         read( model, ch );
         if not inString then begin
            if ch = '(' then begin
               read( model, ch );
               if ch = '*' then
                  skipComment
               else begin
                  tokens[lout, pos] := '(';
                  pos := pos + 1;
               end;
            end;
            if ch = '{' then
               skipComment
         end;
         if ch = '''' then begin
            inString := not inString;
         end;
         if not inString then ch := lcase[ch];
         tokens[lout, pos] := ch;
         pos := pos + 1;
      end; {if eoln( model ) }

   end; {while}
   nrLines := lout;

end; { readSource }


{  advance one token, remove/replace it or add extra token,
   write new mutant program
   on entry:
   mline, mpos = last pos of previous token
   on exit:
   mline, mpos = where to start next time
}
function createMutant(var mline, mpos : integer): boolean;
const
   n        = 9;                {length of keyword}
   blanks   =  '         ';

#if defined ACTION
   action = ACTION;
#else
   action = remove;
#endif

type
   tokStr = packed array[1..n] of char;

var
   i,j        : integer;
   ch         : char;
   more       : boolean;
   s          : tokStr;
   nxtmline   : integer;

{ move to next token, skip whitespace following pos i,
 mline is maintained as necessary
 return pos of next token}
function skip(i : integer): integer;
var
   ch : char;
begin
   ch := ' ';
   while (ch <= ' ') and more do begin;
      i := i+1;
      ch := tokens[mline, i];
      //writeln(__LINE__, ': ch is ''', ch, '''');
      if ch = chr(0) then begin
         if mline = nrLines then begin
            more := false;
         end
         else begin
            i := 0;
            mline := mline + 1;
         end;
         writeln( mutantSource );
      end;
   end; {while}
   //writeln(__LINE__, ': ch is ''', ch, '''');
   skip := i;
end; { skip }


{peek ahead,
 return s = next token if it is alphabetic
 mpos is pos of char before start of search, this char isn't trailing null
}
procedure peek(var s : tokStr);
var
   i,j,l : integer;
begin
   l := mline;
   i := mpos+1;
   s := blanks;
   while (l <= nrLines) and (tokens[l,i] <= ' ') do begin
      if tokens[l,i] = chr(0) then begin
         l := l+1;
         i := 1;
      end
      else
         i := i+1;
   end;

   if l <= nrLines then begin
      j := 1;
      while tokens[l,i] in ['a'..'z'] do begin
         if j <= n then s[j] := tokens[l,i];
         i := i+1;
         j := j+1;
      end;
   end; {if}

end; {peek}


begin {createMutant}
   more := true;
   rewrite( mutantSource );

   write( mutantSource, '{ mutant code - token at line ', mline+1:1, ', pos ', mpos:1 );
   case action of
     add    : writeln( mutantSource, ' added }' );
     remove : writeln( mutantSource, ' removed }' );
     replace: writeln( mutantSource, ' replaced }' );
   end;

   {copy up to mline, mpos}
   //writeln(__LINE__);
   for j := 1 to mline-1 do begin
      writeln( mutantSource, tokens[j] );
   end;
   for j := 1 to mpos do begin
      write( mutantSource, tokens[mline, j]);
   end;
   //writeln(__LINE__);

   {skip to next token}
   i := skip(mpos);

   {i is pos of next token}

   {act on next token}
   if more then begin

      mpos := -1;
      nxtmline := -1;
      s := blanks;


      //writeln(__LINE__);
      case action of
        add    : ;
        remove : write( mutantSource, ' {remove  ***** ' );
        replace: write( mutantSource, ' {replace ***** ' );
      end;
      ch := tokens[mline, i];
      write( mutantSource, ch );

      if ch = '''' then begin
         {remove string token}
         //write('string -->', ch);
         repeat
            i := i+1;
            ch := tokens[mline, i];
            write( mutantSource, ch );
            //write( ch );
            if (ch = '''') and (tokens[mline, i+1] = '''') then begin
               {quote char twice}
               write( mutantSource, '''' );
               i := i+1;
               ch := '?';   {dummy assign to prevent loop terminating}
            end;
         until ch in ['''', chr(0)];  {check against null in case string has no closing "'" char}
         //writeln('<--');
      end
      else if ch = '(' then begin
         parenCount := parenCount+1;
         //writeln('line ', mline:1, ': parenCount is ', parenCount:1);
      end
      else if ch = ')' then begin
         parenCount := parenCount-1;
         //writeln('line ', mline:1, ': parenCount is ', parenCount:1);
      end
      else if ch in ['0'..'9', 'a'..'z'] then begin
         s[1] := ch;
         j := 1;
         while tokens[mline, i+1] in ['0'..'9', 'a'..'z'] do begin
            i := i+1; j := j+1;
            ch := tokens[mline, i];
            write( mutantSource, ch );
            if j <= n then s[j] := ch;
         end;
         {if token so far = "var"/"not"/"packed", then include next token as well}
         //write(' s is ''', s, '''');
         if ((s = 'var      ') and (parenCount > 0)) or
            (s = 'not      ') or (s = 'packed   ')
            then begin
               i := skip(i);
               write( mutantSource, ' ' );
               while tokens[mline, i] in ['0'..'9', 'a'..'z'] do begin
                  ch := tokens[mline, i];
                  write( mutantSource, ch );
                  i := i+1;
                  //trap := true;
               end;
               i := i-1;
            end;
      end
      else if (ch = ';') then begin
         {removing ';' before 'end' or 'until' doesn't test compiler}
         mpos := i;
         peek(s);
         if (action = remove) and ((s = 'end      ') or (s = 'until    ')) then begin
            nxtmline := mline;
            i := skip(i);
            ch := tokens[mline,i];
            assert( ch in (['e', 'u']) );
            if ch = 'e' then begin
               write( mutantSource, 'end' );
               i := i+2;
            end
            else begin
               write( mutantSource, 'until' );
               i := i+4;
            end;
         end; {if "end" or "until"}
      end
      else if (ch = '>') and (tokens[mline, i+1] = '=') then begin
         i := i+1;
         write( mutantSource, '=' );
      end
      else if (ch = '<') and (tokens[mline, i+1] in ['=', '>']) then begin
         i := i+1;
         write( mutantSource, tokens[mline, i] );
      end;

      if action in [replace, remove] then begin
         write( mutantSource, ' ***** }' );
      end;

      {i is last char of removed token}

      {mpos is where we start next time round}
      if mpos < 0 then mpos := i;

      //writeln(__LINE__);

      if action in [add, replace] then begin
         if action = add then
            write( mutantSource, ' {add --> } ' )
         else
            write( mutantSource, ' { --> } ' );

         {addToken}
         assert( ord(othersy) = 48 );
         case rand mod ord(othersy) of
            0 {ident}       : write( mutantSource, 'uuz99');
            1 {intconst}    : write( mutantSource, '123');
            2 {realconst}   : write( mutantSource, '45.56e12');
            3 {stringconst} : write( mutantSource, '''stringconst''');
            4 {notsy}       : write( mutantSource, 'not');
            5 {mulop}       : write( mutantSource, '*');
            6 {addop}       : write( mutantSource, '+');
            7 {relop}       : write( mutantSource, '=');
            8 {lparent}     : write( mutantSource, '(');
            9 {rparent}     : write( mutantSource, '}');
           10 {lbrack}      : write( mutantSource, '[');
           11 {rbrack}      : write( mutantSource, ']');
           12 {comma}       : write( mutantSource, ',');
           13 {semicolon}   : write( mutantSource, ';');
           14 {period}      : write( mutantSource, '.');
           15 {arrow}       : write( mutantSource, '^');
           16 {colon}       : write( mutantSource, ':');
           17 {becomes}     : write( mutantSource, ':=');
           18 {range}       : write( mutantSource, '..');
           19 {labelsy}     : write( mutantSource, 'label');
           20 {constsy}     : write( mutantSource, 'const');
           21 {typesy}      : write( mutantSource, 'type');
           22 {varsy}       : write( mutantSource, 'var');
           23 {funcsy}      : write( mutantSource, 'func');
           24 {progsy}      : write( mutantSource, 'prog');
           25 {procsy}      : write( mutantSource, 'proc');
           26 {setsy}       : write( mutantSource, 'set');
           27 {packedsy}    : write( mutantSource, 'packed');
           28 {arraysy}     : write( mutantSource, 'array');
           29 {recordsy}    : write( mutantSource, 'record');
           30 {filesy}      : write( mutantSource, 'file');
           31 {beginsy}     : write( mutantSource, 'begin');
           32 {ifsy}        : write( mutantSource, 'if');
           33 {casesy}      : write( mutantSource, 'case');
           34 {repeatsy}    : write( mutantSource, 'repeat');
           35 {whilesy}     : write( mutantSource, 'while');
           36 {forsy}       : write( mutantSource, 'for');
           37 {withsy}      : write( mutantSource, 'with');
           38 {gotosy}      : write( mutantSource, 'goto');
           39 {endsy}       : write( mutantSource, 'end');
           40 {elsesy}      : write( mutantSource, 'else');
           41 {untilsy}     : write( mutantSource, 'until');
           42 {ofsy}        : write( mutantSource, 'of');
           43 {dosy}        : write( mutantSource, 'do');
           44 {tosy}        : write( mutantSource, 'to');
           45 {downtosy}    : write( mutantSource, 'downto');
           46 {thensy}      : write( mutantSource, 'then');
           47 {nilsy}       : write( mutantSource, 'nil');
         end;
         write(  mutantSource, ' { <-- } ');
         //if action = add then trap := true;
      end;

      {copy out remainder of source}
      i := i+1;
      while tokens[mline, i] <> chr(0) do begin
         write(mutantSource, tokens[mline, i]);
         i := i+1;
      end;
      writeln( mutantSource );
      //writeln(__LINE__);
      for i := mline + 1 to nrLines do
         writeln( mutantSource, tokens[i] );
      //writeln(__LINE__);

      if nxtmline > 0 then mline := nxtmline;

   end; {not more}

   //writeln(__LINE__);
   reset(mutantSource);

   createMutant := more;
   //writeln(__LINE__, ', createMutant returns ', more);
end; { createMutant }


procedure writeMutant;
var
   i : integer;
begin
   rewrite(mutantSource);
   for i := 1 to nrLines do
      writeln( mutantSource, tokens[i] );
   reset(mutantSource);
end; { writeMutant }


{check the list file of the compiled mutant}
procedure check;
const
   errStr = 'Errors in program:';
   len    =  18; // length of errStr

var
   n      : integer;
   i      : integer;
   ch     : char;
   chkStr : packed array[1..len] of char;

begin
   chkStr := errStr;
   reset( listFile );
   if eof( listFile ) then begin
      writeln('list file is empty');
      exit(1);
   end; {if}
   n := -1;
   while not eof( listFile ) do begin
      if eoln (listFile ) then begin
         readln( listFile );
      end
      else begin
         {does this line contain error count?}
         i := 0;
         while i < len do begin
            if eoln( listFile ) then
               i := maxint
            else begin
               i := i + 1;
               read( listFile, ch );
               //write(ch);
               if ch <> chkStr[i] then
                  i := maxint;
            end;
         end; {while i < len}
         if i = len then begin
            {found number of errors}
            read( listFile, n );
            //writeln( 'mutant(', lnr:1, ',', pos:1, ') has ', n:1, ' compilation errors' );
            assert(n >= 0);
            if n = 0 then begin
               writeln(lnr:5, ': ', tokens[lnr]);
               writeln('^':pos+7);
               //readln;
            end;
         end;
      end; {if}
   end; {while}

   if n = -1 then begin
      writeln( 'mutant(', lnr:1, ',', pos:1, ') error count not found - maybe crashed?');
      readln;
   end;
end; { check }


begin

   if system(' ') = 0 then begin
      writeln('no shell to run compile command - quitting');
      exit(1);
   end;

#if defined ACTION
   if ACTION in [add, replace] then
      initAdd;
   write('action is ' );
   case ACTION of
     add    : write( 'add' );
     remove : write( 'remove' );
     replace: write( 'replace' );
   end;
#else
   write('action is remove' );
#endif
   writeln(', cmdStr is ''', cmdStr, '''');

   readSource;
//   writeMutant;

   trap := false;
   lnr := 1; pos := 0;    parenCount := 0;
   while createMutant(lnr, pos) do begin
      assert(parenCount >= 0);
      //write('created mutant nr (', lnr:1, ',', pos, '),   OK?');  readln;

      {compile the mutant}
      syStatus := system( cmdStr );
      //writeln( ' system returns ', syStatus:1 ) );
      if syStatus <> 0 then begin
         writeln('command status is ', syStatus);
         exit(syStatus);
      end;

      {did the compile command crash?}
      check;

      {this is for debugging}
      if trap then begin
         write('trap>');
         readln;
         trap := false;
      end;
      now := clock;
      //while clock - now < 1000.0 do;

   end; {while createMutant}

   writeln('all mutants done');
end.


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%% end of mutate.pas %%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
