#!/bin/sh

###################################################

# run a set of standard pascal compiler tests

###################################################

if [ -z $P5CDIR ] ; then
    echo "\$P5CDIR is not set, bailing out"
    exit 1;
fi

echo
echo "iso 7185 standard tests"

rm -f conformant conformant.exe conformant.out
$P5CDIR/r conformant | tee conformant.out | awk 'BEGIN { nf=0; np=0; } \
                               /^ PASS/ { np++ }; \
                               /FAIL/ {nf++; print}; \
                        END { print np, "tests passed,", nf==1? "one test": nf " tests", "failed" }'
if ! tail conformant.out | grep -q "tests completed" ; then
    echo "conformant tests not completed"
    printf 'Press RETURN to continue ... '
    read REPLY
fi
if grep -E -e '^c6p[0-9p]+[a-z]+ *unreferenced' conformant.lst ; then
    echo "conformant tests ommitted"
    printf 'Press RETURN to continue ... '
    read REPLY
fi

echo

rm -f ta ta.exe
$P5CDIR/r ta
test -x ta || echo "ta tests failed"

printf 'Press RETURN to continue ... '
read REPLY

cat deviance.pas errorHandling.pas | \
    awk 'BEGIN { FS="[ (]"; i=0; \
                   l[0] = "{ --- DO NOT EDIT THIS FILE --- }" ;} \
        /^{ *TEST +/ { if(i>0) l[i++] = "xxx ERROR xxx" ; \
                       else i=1; name = "d6p10d" }; \
        i>0 { l[i++]=$0}; \
        /^ *program +/ { name=$2 }; \
        /^ *end\./ {n=i; for(i=0; i<n; i++) print l[i] > name ".p"; i=0 }'

if grep -c "xxx ERROR xxx" [de]*[!.]?.p | grep -v "p:0"
then
    grep -c "xxx ERROR xxx" [de]*[!.]?.p | \
        awk -F ":" '$2 != 0 {print $1, "seems to have", $2, "errors"}'
    printf 'Press RETURN to continue ... '
    read REPLY
fi

URE=""
UCE=0
for i in [de][1-9]p[1-9]*[a-z].p; do
    PASFILE=`basename $i .p`
    rm -f $PASFILE $PASFILE.lst $PASFILE.out
    echo -n ' ' | $P5CDIR/r $PASFILE > $PASFILE.out 2>&1
    echo -n "$i: "
    if grep -qF "Errors in program: 0" $PASFILE.lst ; then
        if [ -f $PASFILE.out ] ; then
            awk 'BEGIN {err=1; ln=0} \
               NR==FNR { if( $1~/fatal/ ) ln=$5; next} \
               /!!!/ {if(FNR":" == ln) {print "OK"; err=0} } \
              END { if( ln == 0 ) print "no runtime error"; \
                    else if(err) print "unverified runtime error";
                    exit((ln!=0) && err);
                  }' ./$PASFILE.out ./$PASFILE.p
            if [ $? -ne 0 ] ; then
                URE="$URE $PASFILE"
            fi
        else
            echo "$PASFILE did not produce an output"
            printf 'Press RETURN to continue ... '
            read REPLY
        fi
    else
        rm -f $PASFILE.out
        # compile failed - expected errors are marked with '!!!'
        awk 'BEGIN {e=0; l1=""; } \
             /\*\*\*\* +\^/ && (l1 ~ /!!!/ || l2 ~ /!!!/) { e = 1; } \
             { l2 = l1; l1 = $0; } \
             END { if(!e) print "------ unverified compile error -------"; \
                   else print "OK"; \
                   exit(e)} \
             ' $PASFILE.lst
        if test $? -eq 0 ; then
            less $PASFILE.lst
            UCE=$(($UCE+1))
        fi
    fi
done

if false; then
# TODO: this could be improved, it misses some verifiable errors
echo
echo "`ls [de][1-9]p[1-9]*[a-z].p | wc -w` tests"
echo '---------------'
awk ' BEGIN { n=0; } \
        FNR==1 {e=1;} \
        /program [de][1-9]p[1-9]/ {prog=$3;} \
        $2 ~ /\*\*\*\*/ && e { \
               e=0; \
               if( l !~ /!!!/ )  { n++; \
                 print "program " prog;    \
                 print l; \
                 print; \
                 print "------" } }\
        { l = $0; } \
        END { print "unverified compilation errors found in " n " tests";  } ' \
    [de][1-9]p[1-9]*[a-z].lst
fi

echo
echo "`ls [de][1-9]p[1-9]*[a-z].p | wc -w` tests"
echo
echo '---------------'
awk ' BEGIN { n=0; } \
       /DEVIATES/ { a[n++] = FILENAME ": " $0 }; \
       END {print "deviations found in " n " tests:"; \
       for(i=0;i<n;i++) print a[i]; } ' [de][1-9]p[1-9]*[a-z].out

echo
echo '---------------'
awk ' BEGIN { n=0; } \
       /ERROR NOT DETECTED/ { a[n++] = FILENAME ": " $0 }; \
       END {print n " tests did not detect an error:"; \
       for(i=0;i<n;i++) print a[i]; } ' [de][1-9]p[1-9]*[a-z].out

echo
echo '---------------'
if [ -z "$URE" ]; then
    echo "all runtime errors verified"
else
    echo "unverfied runtime errors: $URE"
fi

echo "unverified compilation errors found in $UCE tests"

#exit


echo '---------------'
for i in *.lst ; do
    if ! grep -q "Errors in program:" $i ; then
        echo "$i did not compile";
    fi;
done

for i in [!x]*.err ; do
    if grep -q ": error:" $i ; then
        echo "bad c code in $i";
    fi;
done

grep -H 'Compiler internal error' *.lst
grep -lH 'No such file or directory' *.out

# for debug version of pcom.pas
# show where memory leaks
echo "found `grep 'new memory at line' *.lst | wc -l` memory leaks"
cat *.lst | grep -H "new memory at line" | sort | uniq


printf 'Press RETURN to continue ... '
read REPLY

if which valgrind > /dev/null 2>&1; then

    MYDIR=$(pwd)
    #echo $MYDIR
    #echo ${MYDIR%/*}
    GENFILE=${MYDIR%/*}/gen

    export $(grep BUILD= $GENFILE)

    echo -n "waiting for $BUILD.t ... "
    while [ ! -x $P5CDIR/$BUILD.t ] ; do sleep 1; done
    echo "$BUILD valgrind tests ..."
    rm -f val-test.vlg
    for i in ta.pas conformant.pas [de]*.p
    do
        printf "\r$i                        "
        valgrind -s --track-origins=yes $P5CDIR/$BUILD.t $i 2>> val-test.vlg > /dev/null
    done
    echo

    awk 'BEGIN {nerr = 0;} \
         /Command:/ {cmd=$4;} \
         /HEAP SUMMARY:/   { \
           getline; if( !/ 0 bytes in 0 blocks/ ) { \
                        nerr++; \
                         print( cmd ": heap error") \
                    } \
           } \
         /ERROR SUMMARY:/ && !/0 errors/  { \
           nerr++; \
           print( cmd ": memory error" ) \
           } \
        END { \
           if( nerr == 0 ) \
              print( "valgrind found no errors"); \
           else if( nerr == 1) \
              print( "valgrind found one error"); \
           else \
              print( "valgrind found",  nerr,  "errors"); \
          } \
        ' val-test.vlg

    read -p 'Press RETURN to continue ... ' REPLY
    echo
fi
